//    Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)
//    Copyright (C) 2008 Chris Kroells
//
//    This file is part of Dvaspi.
//
//    Dvaspi is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Dvapsi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.


package net.coobird.dvaspi;

import java.awt.*;

/**
 * The DataSet class is the representation of a set of data.
 * The data points are stored along with the points that are scaled
 * to the screen, and properties such as colors and viewport
 * is stored as well.
 * @author coobird
 */
public class DataSet
{
	/**
	 * Mode for not showing any point labels.
	 */
	public static final int MODE_SHOW_NONE = 0;
	/**
	 * Mode for displaying labels on selected points.
	 */
	public static final int MODE_SHOW_SELECTED = 1;
	/**
	 * Mode for displaying labels on all points.
	 */
	public static final int MODE_SHOW_ALL = 2;
	
	private String name = "Untitled";
	private String filename = "";
	
	// actual data from file
	public double[] x;
	public double[] y;
	// data scaled to screen coordinates
	public int[] sx;
	public int[] sy;
	public boolean[] selected;

	// Windowing for the DataSet
	// Running calcMinMax() will set this to full windowing
	// Changing these values will change the windowing of the data set
	// in the panel display.
	public double leftX;
	public double rightX;
	public double topY;
	public double bottomY;
	
	// these are the absolute limits to the dataset
	public double absLeftX;
	public double absRightX;
	public double absTopY;
	public double absBottomY;
	
	// Draw settings for the data set	
	public boolean drawLines = true;
	public boolean drawPoints = true;
	public boolean drawXLabel = true;
	public boolean drawYLabel = true;
	
	private int pointDiameter = 4;
	private int pointRadius = 2;

	private int labelMode;
	
	public Color pointColor = Color.black;
	public Color pointSelectedColor = Color.red;
	public Color lineColor = Color.blue;
	public Color labelColor = Color.black;
	
	public double xfactor;
	public double yfactor;
	public int offsetx;
	public int offsety;
	
	
	/**
	 * Sets the name of this data set.
	 * @param s The name to set the data set to.
	 */
	public void setName(String s)
	{
		name = s;
	}
	
	/**
	 * Returns the name of the data set.
	 * @return The name of the data set.
	 */
	public String getName()
	{
		return name;
	}

	/**
	 * Sets the filename of the data set.
	 * In lieu of actually holding the File object of the
	 * data set itself, the DataSet object holds onto the
	 * filename String.
	 * @param s The filename of the data set.
	 */
	public void setFilename(String s)
	{
		filename = s;
	}
	
	/**
	 * Returns the filename of the data set.
	 * @return The filename of the data set.
	 */
	public String getFilename()
	{
		return filename;
	}

	/**
	 * Sets the data points of the data set.
	 * @param x An array containing the values of the independent axis.
	 * @param y An array containing the values of the dependent axis.
	 */
	public void setData(double[] x, double[] y)
	{
		this.x = x;
		this.y = y;
		
		this.selected = new boolean[this.x.length];
		for (int i = 0; i < this.x.length; i++)
			this.selected[i] = false;
	}
	
	/**
	 * Sets the data points of the data set scaled to the screen.
	 * @param sx An array containing the independent axis values scaled to the screen.
	 * @param sy An array containing the dependent axis values scaled to the screen.
	 */
	public void setScaledData(int[] sx, int[] sy)
	{
		this.sx = sx;
		this.sy = sy;
	}
	
	/**
	 * Sets the scaling factors and the viewport offset of the
	 * data set. 
	 * @param fx The x-axis scaling factor.
	 * @param fy The y-axis scaling factor.
	 * @param ox The x-axis offset.
	 * @param oy The y-axis offset.
	 */
	public void setFactors(double fx, double fy, int ox, int oy)
	{
		xfactor = fx;
		yfactor = fy;
		offsetx = ox;
		offsety = oy;
	}
	
	/**
	 * Finds the minimum and maximum values of the two dimensions,
	 * then sets the appropriate variables to allow correct display
	 * on a DvaspiPanel.
	 */
	public void calcMinMax()
	{
		leftX = x[0];
		rightX = x[0];
		for (int i = 0; i < x.length; i++)
		{
			if (x[i] > rightX)
				rightX = x[i];
			else if (x[i] < leftX)
				leftX = x[i];
		}
		
		topY = y[0];
		bottomY = y[0];
		for (int i = 0; i < y.length; i++)
		{
			if (y[i] > bottomY)
				bottomY = y[i];
			else if (y[i] < topY)
				topY = y[i];
		}
		
		absLeftX = leftX;
		absRightX = rightX;
		absTopY = topY;
		absBottomY = bottomY;
	}

	/**
	 * Sets the mode for displaying the labels.
	 * Valid options are all, selected, none, constants which
	 * are defined in the DataSet class.
	 * @param labelMode The mode for displaying labels.
	 */
	public void setLabelMode(int labelMode)
	{
		this.labelMode = labelMode;
	}

	/**
	 * Returns the current label mode.
	 * Refer to the DataSet class for the constants.
	 * @return The current label mode.
	 */
	public int getLabelMode()
	{
		return labelMode;
	}

	/**
	 * Returns the diameter for the data point displayed.
	 * @return The diameter of the data point.
	 */
	public int getPointDiameter()
	{
		return pointDiameter;
	}

	/**
	 * Sets the point diameter of a data point.
	 * @param pointDiameter The diameter of the data point.
	 */
	public void setPointDiameter(int pointDiameter)
	{
		this.pointDiameter = pointDiameter;
	}

	/**
	 * Returns the radius for the data point displayed.
	 * @return The radius of the data point.
	 */
	public int getPointRadius()
	{
		return pointRadius;
	}

	/**
	 * Sets the point radius of a data point.
	 * @param pointRadius The radius of the data point.
	 */
	public void setPointRadius(int pointRadius)
	{
		this.pointRadius = pointRadius;
	}
}