//    Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)
//    Copyright (C) 2008 Chris Kroells
//
//    This file is part of Dvaspi.
//
//    Dvaspi is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Dvapsi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.


package net.coobird.dvaspi;

import java.io.*;
import java.util.*;

// TODO
// 1. Perhaps a feature to stop opening a file if too many lines
// are not recognized as data?
// 2. a way to support progress bars (in terms of "bytes read / bytes of file")
//   --> spawn a new thread for getData(File,DataSet,int)?

/**
 * Data processing class of Dvapsi.
 * @author coobird
 * 
 */
public class Dvaspi
{
	/**
	 * Undefined data type.
	 */
	public static final int UNDEFINED = -1;
	/**
	 * Comma-separated value (CSV).
	 */
	public static final int COMMA_SEPARATED = 0;
	/**
	 * Tab-delimited text.
	 */
	public static final int TAB_DELIMITED = 1;
	
	public static PrintStream out = System.out;
	
	/**
	 * Sets the PrintStream to which messages are written to.
	 * @param ps PrintStream object.
	 */
	public static void setPrintStream(PrintStream ps)
	{
		out = ps;
	}
	
	/**
	 * Reads and gets data points from a file and stores in a data set.
	 * @param f File to retrieve data points from.
	 * @param ds DataSet object to store the data points to.
	 * @param type The type of the data being opened.
	 * @return Returns true if data was successfully retrieve, false if not.
	 * False will be returned when an exception is thrown during processing.
	 */
	public static boolean getData(File f, DataSet ds, int type)
	{
		// set the delimiting character
		String splitChar;
		if (type == COMMA_SEPARATED)
			splitChar = ",";
		else if (type == TAB_DELIMITED)
			splitChar = "\t";
		else
			splitChar = " ";
		
		// initialize the data input vectors
		Vector<Double> vx = new Vector<Double>();
		Vector<Double> vy = new Vector<Double>();
		
		//out.print("reading file: " + f.getName() + "...");
		try
		{
			// read in the file
			BufferedReader br = new BufferedReader(new FileReader(f));
			String line = "";
			
			// while there's stuff to read, get that data
			while(br.ready())
			{
				line = br.readLine();
				String[] pair = line.split(splitChar);
				if (pair.length == 2)
				{
					vx.add(Double.parseDouble(pair[0]));
					vy.add(Double.parseDouble(pair[1]));
				}
			}
			
			// done reading file
			out.println("done");
			
			// move the vector data to array
			Object[] arrayx = vx.toArray();
			Object[] arrayy = vy.toArray();
			
			double[] x = new double[arrayx.length];
			double[] y = new double[arrayy.length];
			
			for (int i = 0; i < arrayx.length; i++)
			{
				x[i] = ((Double)(arrayx[i])).doubleValue();
				y[i] = ((Double)(arrayy[i])).doubleValue();
			}
			
			// Sets the data values and properties for the DataSet
			ds.setName(f.getName());
			ds.setFilename(f.getName());
			ds.setData(x, y);
			ds.calcMinMax();
		}
		catch (Exception e)
		{
			e.printStackTrace(out);
			return false;
		}
		
		return true;
	}
	
	/**
	 * Reads and gets data points from a file and stores in a data set.
	 * This is a convenience method for getData(File,DataSet,int)
	 * where the type of input is determined by the extension
	 * of the file being opened.
	 * @param f File to retrieve data points from.
	 * @param ds DataSet object to store the data points to.
	 * @return Returns true if data was successfully retrieve, false if not.
	 * False will be returned when an exception is thrown during processing.
	 */
	public static boolean getData(File f, DataSet ds)
	{
		if (f.getName().toLowerCase().endsWith("csv"))
			return getData(f, ds, Dvaspi.COMMA_SEPARATED);
		else if (f.getName().toLowerCase().endsWith("txt"))
			return getData(f, ds, Dvaspi.TAB_DELIMITED);
		else
			return getData(f, ds, Dvaspi.UNDEFINED);
	}
}