//    Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)
//    Copyright (C) 2008 Chris Kroells
//
//    This file is part of Dvaspi.
//
//    Dvaspi is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Dvapsi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.


package net.coobird.dvaspi;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.event.*;

import java.io.*;
import java.util.*;

// TODO
// 2. Windowing capabilities
// 8. Allow mulitple zoom-in? --> have multiple data sets selected and allow zooming
//    --> this is a feature that makes sense if data sets have same viewports
// 11. Move the MenuBar to another class?
//		* Requires a rewrite of actionPerformed
// 12. Select Range and Select Region (for version 0.6)
// 13. Show axes
// 14. Store preferences (window position, dimensions, etc; for ver 0.6)


/**
 * The graphical interface for Dvaspi.
 * @author coobird
 * 
 */
public class DvaspiGui implements ActionListener, ListSelectionListener, MessageListener
{
	/**
	 * Version string.
	 */
	private final String version = "0.5";
	
	/**
	 * Number of points before prompting to ask if user wants to enable
	 * Fast Graphics mode.
	 */
	private final int LENGTH_MANYPOINTS = 5000; 
		
	private File fcCurDir;
		
	private JMenuBar menuBar;
	
	private JMenu fileMenu;
	private JMenuItem fileOpenMenuItem;
	private JMenuItem fileExportMenuItem;
	private JMenuItem fileExportSpecialMenuItem;
	private JMenuItem fileExitMenuItem;
		
	private JMenu viewMenu;
	private JCheckBoxMenuItem viewShowPointsMenuItem;
	private JCheckBoxMenuItem viewShowLinesMenuItem;
	
	private JMenu viewColorMenu;
	private JMenuItem viewColorBackgroundMenuItem;
	
	private JMenu viewDockMenu;
	private JMenuItem viewDockPropertiesMenuItem;
	
	private JMenu modeMenu;
	private JRadioButtonMenuItem modeNoneMenuItem;
	private JRadioButtonMenuItem modeSelectMenuItem;
	private JRadioButtonMenuItem modeProbeMenuItem;
	private JRadioButtonMenuItem modeClosestMenuItem;
	private JRadioButtonMenuItem modeIntegrateMenuItem;
	private JRadioButtonMenuItem modeZoomInMenuItem;
	private JRadioButtonMenuItem modeZoomOutMenuItem;
	private JRadioButtonMenuItem modeZoomCenterMenuItem;
	private ButtonGroup modeMenuButtonGroup;
	
	private JMenu processMenu;
	private JMenuItem processIntegrateMenuItem;
	
	private JMenu performanceMenu;
	private JCheckBoxMenuItem performanceFancyGraphics;
	private JCheckBoxMenuItem performanceFastGraphics;
	
	
	private JMenu helpMenu;
	private JMenuItem helpAboutMenuItem;
	
	private DvaspiPanel ip;
		
	private boolean debug = false;
	private JFrame mainFrame;
	private JDialog propertiesDialog;
	
	
	private JTabbedPane tp;
	
	private JPanel dataSetPanel;
	private JList dataSetList;
	private JScrollPane dataSetListScrollPane;
	private DefaultListModel dataSetListModel;
	
	private JButton dataSetRemoveButton;
	private JButton dataSetFitToViewportButton;
	private JButton dataSetMatchViewportsButton;
		
	private WrapArray<Color> colorForLine;
		
	private DvaspiWindowPanel dwp;
	private JPanel dwpPanel;
	
	private DvaspiConsole dc;
	private DvaspiStatusBar statusBar;
	private DvaspiSelectionTable dst;
	private DvaspiPropertiesPanel dpp;
		
	/**
	 * Generates the Dvaspi graphical user interface.
	 *
	 */
	public void makeGUI()
	{
		// init main window
        mainFrame = new JFrame("Coobird's Data Viewing and Selecting Program with Integration");
        mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        mainFrame.setSize(600,400);
        mainFrame.getContentPane().setLayout(new BorderLayout());
		
		// make menu bar
        makeMenuBar();
		mainFrame.setJMenuBar(menuBar);
				
		// init int panel
		ip = new DvaspiPanel();

		// Sets the performance settings
		setFastGraphics(false);
		setFancyGraphics(true);

		// Selection Table
		dst = new DvaspiSelectionTable();
		dst.addMessageListener(this);
		
		// DataSet list being set up
		dataSetListModel = new DefaultListModel();
		dataSetList = new JList(dataSetListModel);
		dataSetListScrollPane = new JScrollPane(dataSetList);
		dataSetList.setCellRenderer(new DataSetCellRenderer());
		dataSetList.addListSelectionListener(this);
		
		// Buttons added to the Data Set tab
		dataSetRemoveButton = new JButton("Remove");
		dataSetFitToViewportButton = new JButton("Fit to viewport");
		dataSetMatchViewportsButton = new JButton("Match viewports");

		JPanel dataSetButtonsPanel = new JPanel(new GridLayout(0, 1));
		dataSetButtonsPanel.add(dataSetRemoveButton);
		dataSetButtonsPanel.add(dataSetFitToViewportButton);
		dataSetButtonsPanel.add(dataSetMatchViewportsButton);
		dataSetRemoveButton.addActionListener(this);
		dataSetFitToViewportButton.addActionListener(this);
		dataSetMatchViewportsButton.addActionListener(this);
		// Set the buttons to disabled state
		setDataSetButtonState(false);
		
		dataSetPanel = new JPanel(new BorderLayout());
		dataSetPanel.setName("Data Sets");
		dataSetPanel.add(dataSetListScrollPane, BorderLayout.CENTER);
		dataSetPanel.add(dataSetButtonsPanel, BorderLayout.SOUTH);
		
		// Make the tabbed and split panes for interface
		tp = new JTabbedPane();
		JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, ip, tp);
		statusBar = new DvaspiStatusBar();
		
		JPanel mainPanel = new JPanel(new BorderLayout());
		mainPanel.add(statusBar, BorderLayout.SOUTH);
		mainPanel.add(sp, BorderLayout.CENTER);
		
		// Add DataSet panel and Selection Table panel to tabbed pane
		tp.add(dataSetPanel);
		tp.add(dst);
		
		// Set up the Window panel and adds to tabbed pane
		dwp = new DvaspiWindowPanel();
		dwpPanel = new JPanel(new BorderLayout());
		dwpPanel.add(dwp, BorderLayout.CENTER);
		dwpPanel.setName("Window");
		tp.add(dwpPanel);
		
		// Split pane initialization
		sp.setOneTouchExpandable(true);
		sp.setContinuousLayout(true);
		sp.setResizeWeight(0.5);
		sp.setDividerSize(10);
		ip.setMinimumSize(new Dimension(200,0));
		
		// set the mode to "nothing"
		modeNoneMenuItem.setSelected(true);
		ip.setMode(DvaspiPanel.MODE_NONE);
		
		// set to show points and lines
		viewShowPointsMenuItem.setSelected(true);
		viewShowLinesMenuItem.setSelected(true);
		
		// Colors used for the DataSets in dataSetList
		colorForLine = new WrapArray<Color>();
		colorForLine.add(Color.blue);
		colorForLine.add(Color.red);
		colorForLine.add(Color.cyan);
		colorForLine.add(Color.green);
		colorForLine.add(Color.yellow);	
		colorForLine.add(Color.magenta);
		
		dc = new DvaspiConsole();
		dc.appendln("Welcome to Coobird's Dvaspi");
		dc.appendln("---------------------------\n");
		dc.setName("Console");
		tp.add(dc);
		
		ip.addActionListener(this);
		ip.addMessageListener(this);
		
		dpp = new DvaspiPropertiesPanel();
		dpp.addMessageListener(this);
		tp.add(dpp);
		
		makePropertiesDialog();
		
		dpp.updatePropertiesTable(null);
		dst.updateSelectedTable(null);
		
		mainFrame.setContentPane(mainPanel);
		mainFrame.setVisible(true);
		mainFrame.validate();
	}
	
	/**
	 * Builds the menu bar
	 */
	private void makeMenuBar()
	{
		// make menu bar
		menuBar = new JMenuBar();
		
		fileMenu = new JMenu("File");
		fileOpenMenuItem = new JMenuItem("Open...");
		fileOpenMenuItem.addActionListener(this);
		fileExportMenuItem = new JMenuItem("Export As...");
		fileExportMenuItem.addActionListener(this);
		fileExportSpecialMenuItem = new JMenuItem("Export Special...");
		fileExportSpecialMenuItem.addActionListener(this);
		fileExitMenuItem = new JMenuItem("Exit");
		fileExitMenuItem.addActionListener(this);
		fileMenu.add(fileOpenMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(fileExportMenuItem);
		fileMenu.add(fileExportSpecialMenuItem);
		fileMenu.addSeparator();
		fileMenu.add(fileExitMenuItem);
		
		viewMenu = new JMenu("View");
		viewShowPointsMenuItem = new JCheckBoxMenuItem("Points");
		viewShowPointsMenuItem.addActionListener(this);
		viewShowLinesMenuItem = new JCheckBoxMenuItem("Lines");
		viewShowLinesMenuItem.addActionListener(this);
		
		viewColorMenu = new JMenu("Colors");
		viewColorBackgroundMenuItem = new JMenuItem("Background...");
		viewColorBackgroundMenuItem.addActionListener(this);		
		viewColorMenu.add(viewColorBackgroundMenuItem);
		
		viewDockMenu = new JMenu("Docking");
		viewDockPropertiesMenuItem = new JMenuItem("Undock Properties");
		viewDockPropertiesMenuItem.addActionListener(this);
		viewDockMenu.add(viewDockPropertiesMenuItem);
		
		viewMenu.add(viewShowPointsMenuItem);
		viewMenu.add(viewShowLinesMenuItem);
		viewMenu.addSeparator();
		viewMenu.add(viewColorMenu);
		viewMenu.add(viewDockMenu);
		
		modeMenu = new JMenu("Mode");
		modeNoneMenuItem = new JRadioButtonMenuItem("Nothing");
		modeNoneMenuItem.addActionListener(this);
		modeSelectMenuItem = new JRadioButtonMenuItem("Select");
		modeSelectMenuItem.addActionListener(this);
		modeProbeMenuItem = new JRadioButtonMenuItem("Probe");
		modeProbeMenuItem.addActionListener(this);
		modeClosestMenuItem = new JRadioButtonMenuItem("Closest Point");
		modeClosestMenuItem.addActionListener(this);
		modeIntegrateMenuItem = new JRadioButtonMenuItem("Integration");
		modeIntegrateMenuItem.addActionListener(this);
		modeZoomInMenuItem = new JRadioButtonMenuItem("Zoom in");
		modeZoomInMenuItem.addActionListener(this);
		modeZoomOutMenuItem = new JRadioButtonMenuItem("Zoom out");
		modeZoomOutMenuItem.addActionListener(this);
		modeZoomCenterMenuItem = new JRadioButtonMenuItem("Zoom center");
		modeZoomCenterMenuItem.addActionListener(this);
		
		modeMenuButtonGroup = new ButtonGroup();
		modeMenuButtonGroup.add(modeNoneMenuItem);
		modeMenuButtonGroup.add(modeSelectMenuItem);
		modeMenuButtonGroup.add(modeProbeMenuItem);
		modeMenuButtonGroup.add(modeClosestMenuItem);
		modeMenuButtonGroup.add(modeIntegrateMenuItem);
		modeMenuButtonGroup.add(modeZoomInMenuItem);
		modeMenuButtonGroup.add(modeZoomOutMenuItem);
		modeMenuButtonGroup.add(modeZoomCenterMenuItem);
		
		modeMenu.add(modeNoneMenuItem);
		modeMenu.add(modeSelectMenuItem);
		modeMenu.add(modeProbeMenuItem);
		modeMenu.add(modeClosestMenuItem);
		modeMenu.add(modeIntegrateMenuItem);
		modeMenu.addSeparator();
		modeMenu.add(modeZoomInMenuItem);
		modeMenu.add(modeZoomOutMenuItem);
		modeMenu.add(modeZoomCenterMenuItem);
		
		processMenu = new JMenu("Process");
		processIntegrateMenuItem = new JMenuItem("Integrate");
		processIntegrateMenuItem.addActionListener(this);
		processIntegrateMenuItem.setEnabled(false);
		processMenu.add(processIntegrateMenuItem);
		
		performanceMenu = new JMenu("Performance");
		performanceFancyGraphics = new JCheckBoxMenuItem("Fancy Graphics");
		performanceFancyGraphics.addActionListener(this);
		performanceFastGraphics = new JCheckBoxMenuItem("Fast Graphics");
		performanceFastGraphics.addActionListener(this);
		performanceMenu.add(performanceFancyGraphics);
		performanceMenu.add(performanceFastGraphics);

		helpMenu = new JMenu("Help");
		helpAboutMenuItem = new JMenuItem("About...");
		helpAboutMenuItem.addActionListener(this);
		helpMenu.add(helpAboutMenuItem);
		
		menuBar.add(fileMenu);
		menuBar.add(viewMenu);
		menuBar.add(modeMenu);
		menuBar.add(processMenu);
		menuBar.add(performanceMenu);
		menuBar.add(helpMenu);
	}
	
	/**
	 * Makes the Properties dialog box
	 */
	private void makePropertiesDialog()
	{
		propertiesDialog = new JDialog(mainFrame, "Properties");
		propertiesDialog.setLocation(mainFrame.getX() + mainFrame.getWidth(), mainFrame.getY());
		propertiesDialog.setSize(200, mainFrame.getHeight());
		propertiesDialog.addWindowListener(new WindowListener() {

			public void windowClosing(WindowEvent arg0)
			{
				// Docks the DvaspiPropertiesPanel instance to the
				// tabbed pane
				propertiesDialog.getContentPane().remove(dpp);
				tp.add(dpp);
				viewDockPropertiesMenuItem.setText("Undock Properties");
			}
			
			public void windowClosed(WindowEvent arg0) {}
			public void windowActivated(WindowEvent arg0) {}
			public void windowDeactivated(WindowEvent arg0) {}
			public void windowDeiconified(WindowEvent arg0) {}
			public void windowIconified(WindowEvent arg0) {}
			public void windowOpened(WindowEvent arg0) {}
			
		});
	}
	
	/**
	 * Update the list of DataSets that are displayed in the DataSet JList
	 *
	 */
	private void doListUpdate()
	{
		int index = dataSetList.getSelectedIndex();
		
		dataSetList.setSelectedIndex(-1);
		dataSetListModel.removeAllElements();
		
		for (Enumeration<DataSet> e = ip.getDataSetVector().elements(); e.hasMoreElements(); )
			dataSetListModel.addElement((DataSet)e.nextElement());
		
		dataSetList.setSelectedIndex(index);
		
		dwp.setDataSetVector(ip.getDataSetVector());
	}
	
	/**
	 * Returns the file extension of a given filename.
	 * @return The file extension.
	 */
	private String getExtension(String filename)
	{
		int index = filename.lastIndexOf('.');
		
		if (index == -1)
			return null;
		
		return filename.substring(index+1);
	}

		
	/**
	 * Performs the image export function, including displaying the 
	 * Save dialog box and populating the supported image formats.
	 * @param imageWidth
	 * @param imageHeight
	 */
	private void showSaveFileChooser(int imageWidth, int imageHeight)
	{
		// prepare Save dialog
		JFileChooser fc = new JFileChooser();
		if (fcCurDir != null)
			fc.setCurrentDirectory(fcCurDir);
		
		// Gets the supported image output formats
		String[] formats = javax.imageio.ImageIO.getWriterFormatNames();
				
		Vector<CustomFileFilter> v = new Vector<CustomFileFilter>();
		CustomFileFilter ff;
		
		// Add the supported formats to the MyFileFilter
		for (int i = 0 ; i < formats.length; i++)
		{
			if (formats[i].equals("png"))
			{
				ff = new CustomFileFilter();
				ff.addExtension("png");
				ff.setType("png");
				ff.setDescription("Portable Network Graphics (png)");
				v.add(ff);
			}
			else if (formats[i].equals("jpg"))
			{
				ff = new CustomFileFilter();
				ff.addExtension("jpg");
				ff.addExtension("jpeg");
				ff.setType("jpeg");
				ff.setDescription("JPEG image (jpg, jpeg)");
				v.add(ff);
			}
		}
		
		Object ofilter[] = v.toArray();
		CustomFileFilter filter[] = new CustomFileFilter[v.size()];
		
		// Removes the "All Files" filter from Save dialog
		fc.setAcceptAllFileFilterUsed(false);
		
		// Adds the filters for the supported formats
		for (int i = 0; i < ofilter.length; i++)
		{
			filter[i] = (CustomFileFilter)ofilter[i];
			fc.addChoosableFileFilter((CustomFileFilter)ofilter[i]);
		}

		// Show Save dialog
		int action = fc.showSaveDialog(mainFrame);
		
	    // after the "Save" button is pressed
		if (action == JFileChooser.APPROVE_OPTION)
	    {
			// Check if file exists
			// If file exists, then ask user to overwrite or not
			if (fc.getSelectedFile().exists())
			{
				int response = JOptionPane.showConfirmDialog(mainFrame, 
						fc.getSelectedFile() + "already exists.\n" +
						"Replace existing file?",
						"Export", JOptionPane.YES_NO_OPTION,
						JOptionPane.WARNING_MESSAGE);
	    		
		    	// If user says not to overwrite
				if (response == JOptionPane.NO_OPTION)
		    	{
		    		JOptionPane.showMessageDialog(mainFrame,
		    			"Image export was cancelled.", "Export cancelled.",
		    			JOptionPane.INFORMATION_MESSAGE);
		    		
		    		// Store current directory for next time use
		    		fcCurDir = fc.getCurrentDirectory();
		    		
		    		return;
		    	}
			}
			
	    	// Actually save file
			for (int i = 0; i < filter.length; i++)
	    	{
	    		if (filter[i] == fc.getFileFilter())
	    		{
	    			try
	    			{
	    				// Determine file export type from extension
				    	if (filter[i].isExtension(getExtension(fc.getSelectedFile().toString())))
				    	{
					    	ip.exportImage(filter[i].getType(), fc.getSelectedFile(), imageWidth, imageHeight);
				    		dc.appendln("Saved " + fc.getSelectedFile() + " as " + filter[i].getDescription() + " file.");
					    	
				    	}
				    	// If extension missing, then add the extension.
				    	// Type is determined from which type is selected.
				    	else
				    	{
				    		String filename = fc.getSelectedFile().toString() + "." + filter[i].getType();
				    		ip.exportImage(filter[i].getType(), new File(filename), imageWidth, imageHeight);
				    		dc.appendln("Saved " + filename + " as " + filter[i].getDescription() + " file.");
				    	}
	    			}
	    			catch (Exception ex)
	    			{
	    				dumpExceptionInfo(ex);
	    			}
	    		}
	    	}
	    	
	    	// Store the current directory for use next time.
	    	fcCurDir = fc.getCurrentDirectory();
	    }
	}

	
	/**
	 * Displays an modal dialog box that contains an error message.
	 * @param msg Message to be displayed in the message box.
	 * @param title Title of the message box.
	 */
	private void showErrorMessage(String msg, String title)
	{
		JOptionPane.showMessageDialog(mainFrame, msg, title, JOptionPane.ERROR_MESSAGE);
	}
	
	/**
	 * Displays an modal dialog box that contains an error message.
	 * The error message will also display the message contained in the Exception
	 * which caused the error.
	 * @param msg Message to be displayed in the message box.
	 * @param title Title of the message box.
	 * @param ex Exception.
	 */
	private void showErrorMessage(String msg, String title, Exception ex)
	{
		JOptionPane.showMessageDialog(mainFrame, msg + "\n" + ex.getMessage(), title, JOptionPane.ERROR_MESSAGE);
		dumpExceptionInfo(ex);
	}
	
	/**
	 * Processes the action for the dataSetRemoveButton.
	 */
	private void processActionDataSetRemoveButton()
	{
		// If no dataset is selected, then throw up an error message
		if (dataSetList.getSelectedIndex() == -1)
		{
			showErrorMessage("No data set is selected for removal.", "Error");
		}
		else if (ip.getMode() != DvaspiPanel.MODE_NONE)
		{
			showErrorMessage("Cannot remove data set during processing.", "Error");
		}
		else
		{
			Object[] ds = dataSetList.getSelectedValues();
			
			for (int i = 0; i < ds.length; i++)
			{
				ip.removeDataSet((DataSet)ds[i]);
				doListUpdate();
				ip.drawOnce();
			}
		}
	}
	
	/**
	 * Processes the action for the dataSetFitToViewportButton.
	 */
	private void processActionDataSetFitToViewportButton()
		{
			// If no dataset is selected, then throw up an error message
			if (dataSetList.getSelectedIndex() == -1)
			{
				showErrorMessage("No data set is selected.", "Error");
			}
			else
			{
				Object[] ds = dataSetList.getSelectedValues();
				
				for (int i = 0; i < ds.length; i++)
				{
					DataSet dsi = (DataSet)ds[i];
					
					dsi.leftX = dsi.absLeftX;
					dsi.rightX = dsi.absRightX;
					dsi.topY = dsi.absTopY;
					dsi.bottomY = dsi.absBottomY;
				}
				
				DataSet dss = (DataSet)dataSetList.getSelectedValue();
	
				// Update Properties table
				dpp.updatePropertiesTable(dss);
				ip.drawOnce();
			}
		}

	/**
	 * Processes the action for the dataSetMatchViewportsButton.
	 */
	private void processActionDataSetMatchViewportsButton()
	{
		MatchViewportsDialog d = new MatchViewportsDialog(mainFrame, "Match Viewports", true);
		
		// User cancelled the match viewports dialog
		if (d.response == MatchViewportsDialog.CANCEL_BUTTON)
			return;

		// If no dataset is selected, then throw up an error message
		if (dataSetList.getSelectedIndex() == -1)
		{
			showErrorMessage("No data set is selected.", "Error");
			return;
		}
		else if (dataSetList.getSelectedValues().length <= 1)
		{
			showErrorMessage("Must select two or more data sets.", "Error");
			return;
		}
		
		double minX = 0;
		double maxX = 0;
		double minY = 0;
		double maxY = 0;

		// This section finds the absolute limits of the
		// data sets selected
		Object[] dso = dataSetList.getSelectedValues();
		
		DataSet[] ds = new DataSet[dso.length];

		for (int i = 0; i < dso.length; i++)
			ds[i] = (DataSet)dso[i];

		if (d.type == MatchViewportsDialog.FIT_TO_LARGEST)
		{
			minX = ds[0].absLeftX;
			maxX = ds[0].absRightX;
			minY = ds[0].absTopY;
			maxY = ds[0].absBottomY;

			for (int i = 1; i < ds.length; i++)
			{
				if (ds[i].absLeftX < minX)
					minX = ds[i].absLeftX;
				if (ds[i].absRightX > maxX)
					maxX = ds[i].absRightX;
				if (ds[i].absTopY < minY)
					minY = ds[i].absTopY;
				if (ds[i].absBottomY > maxY)
					maxY = ds[i].absBottomY;
			}
		}
		else if (d.type == MatchViewportsDialog.FIT_TO_ALL)
		{
			minX = ds[0].leftX;
			maxX = ds[0].rightX;
			minY = ds[0].topY;
			maxY = ds[0].bottomY;

			for (int i = 1; i < ds.length; i++)
			{
				if (ds[i].leftX < minX)
					minX = ds[i].leftX;
				if (ds[i].rightX > maxX)
					maxX = ds[i].rightX;
				if (ds[i].topY < minY)
					minY = ds[i].topY;
				if (ds[i].bottomY > maxY)
					maxY = ds[i].bottomY;
			}
		}
		else if (d.type == MatchViewportsDialog.FIT_TO_DATASET)
		{
			DataSet targetDS = d.dataSetToFitTo;
			
			minX = targetDS.leftX;
			maxX = targetDS.rightX;
			minY = targetDS.topY;
			maxY = targetDS.bottomY;
		}
		else if (d.type == MatchViewportsDialog.FIT_TO_DATASET_ABS)
		{
			DataSet targetDS = d.dataSetToFitTo;
			
			minX = targetDS.absLeftX;
			maxX = targetDS.absRightX;
			minY = targetDS.absTopY;
			maxY = targetDS.absBottomY;
		}

		dc.appendln("\nMatched viewports for the following data sets:");
		dc.appendln("----------------------------------------------");

		// Sets the viewport for all data sets
		for (int i = 0; i < ds.length; i++)
		{
			ds[i].leftX = minX;
			ds[i].rightX = maxX;
			ds[i].topY = minY;
			ds[i].bottomY = maxY;
			
			dc.appendln("" + (i+1) + ". " + ds[i].getName());
		}
		dc.appendln("");
		
		// Update the text boxes
		DataSet dss = (DataSet)dataSetList.getSelectedValue();

		// Update the Properties
		dpp.updatePropertiesTable(dss);

		ip.drawOnce();
	}
	
	/**
	 * Processes the action for the Mode Menu.
	 * @param source The menu item that triggered the event.
	 */
	private void processActionModeMenu(Object source)
	{
		// Check that only a single DataSet is selected.
		Object[] ds = dataSetList.getSelectedValues();
					
		if (dataSetList.getSelectedIndex() == -1)
		{
			showErrorMessage("No data set is selected.", "Error");
			modeNoneMenuItem.setSelected(true);
			ip.setMode(DvaspiPanel.MODE_NONE);
			ip.clearIsolatedDS();
			ip.drawOnce();
			return;
		}
		else if (ds.length > 1)
		{
			showErrorMessage("Only one data set can be analyzed at once.", "Error");
			modeNoneMenuItem.setSelected(true);
			ip.setMode(DvaspiPanel.MODE_NONE);
			ip.clearIsolatedDS();
			ip.drawOnce();
			return;
		}
		else
		{
			ip.setIsolatedDS((DataSet)ds[0]);
		}
		
		ip.setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
		// prevent File->Open while processing
		fileOpenMenuItem.setEnabled(false);
		processIntegrateMenuItem.setEnabled(false);
		
		statusBar.setMessage("");
		statusBar.setLocation("");
		
		// What to do for each mode selected in a big if-elseif
		if (source == modeNoneMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_NONE);
			ip.clearIsolatedDS();
			ip.setCursor(Cursor.getDefaultCursor());
			fileOpenMenuItem.setEnabled(true);
			statusBar.setMode("None");
		}
		else if (source == modeSelectMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_SELECT);
			statusBar.setMode("Select");
		}
		else if (source == modeProbeMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_PROBE);
			statusBar.setMode("Probe");
		}
		else if (source == modeClosestMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_CLOSEST);
			statusBar.setMode("Closest point");
		}
		else if (source == modeIntegrateMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_INTEGRATE);
			processIntegrateMenuItem.setEnabled(true);
			statusBar.setMode("Integration");
		}
		else if (source == modeZoomInMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_ZOOM_IN);
			statusBar.setMode("Zoom in");
		}
		else if (source == modeZoomOutMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_ZOOM_OUT);
			statusBar.setMode("Zoom out");
		}
		else if (source == modeZoomCenterMenuItem)
		{
			ip.setMode(DvaspiPanel.MODE_ZOOM_CENTER);
			statusBar.setMode("Zoom center");
		}
		
		ip.drawOnce();
	}
	
	/**
	 * Processes the action for the File->Open, openFileMenuItem.
	 */
	private void processActionOpenFileMenuItem()
	{
		// prepare Open dialog
		JFileChooser fc = new JFileChooser();
		if (fcCurDir != null)
			fc.setCurrentDirectory(fcCurDir);
		
		// Populate supported formats
		CustomFileFilter filter[] = new CustomFileFilter[3];
		filter[0] = new CustomFileFilter();
		filter[0].addExtension("csv");
		filter[0].setDescription("Comma-separated value (csv)");
		filter[1] = new CustomFileFilter();
		filter[1].addExtension("txt");
		filter[1].setDescription("Tab-delimited text (txt)");
		filter[2] = new CustomFileFilter();
		filter[2].addExtension("csv");
		filter[2].addExtension("txt");
		filter[2].setDescription("All supported formats (csv, txt)");
		
		// Sets the choosable file formats to the FileChooser
		for (int i = 0; i < filter.length; i++)
			fc.addChoosableFileFilter(filter[i]);
		
		fc.setMultiSelectionEnabled(true);
		
		// show Open dialog
		int action = fc.showOpenDialog(mainFrame);
		
	    if (action == JFileChooser.APPROVE_OPTION)
	    {
	    	File[] files = fc.getSelectedFiles();
	    	
	    	for (int i = 0; i < files.length; i++)
	    	{
		    	Thread t = new Thread(new OpenFileThread(files[i], new DataSet()));
		    	t.start();
	    	}
	    	
	    	fcCurDir = fc.getCurrentDirectory();
	    }
	}
	
	/**
	 * Sets the enabled state for the Data Set panel buttons.
	 * @param b If true, the buttons are enabled. Else, buttons are disabled.
	 */
	private void setDataSetButtonState(boolean b)
	{
		dataSetRemoveButton.setEnabled(b);
		dataSetFitToViewportButton.setEnabled(b);
		dataSetMatchViewportsButton.setEnabled(b);
		dataSetRemoveButton.setFocusable(b);
		dataSetFitToViewportButton.setFocusable(b);
		dataSetMatchViewportsButton.setFocusable(b);
	}
	
	/**
	 * Method to enable Fast Graphics mode on the
	 * DvapsiPanel, and updates the checked state of the
	 * Performance->Fast Graphics menu item.
	 * @param b If true, Fast Graphics is enabled. Else, disabled.
	 */
	private void setFastGraphics(boolean b)
	{
		ip.setFastGraphics(b);
		performanceFastGraphics.setState(b);
	}

	/**
	 * Method to enable Fancy Graphics mode on the
	 * DvapsiPanel, and updates the checked state of the
	 * Performance->Fancy Graphics menu item.
	 * @param b If true, Fancy Graphics is enabled. Else, disabled.
	 */
	private void setFancyGraphics(boolean b)
	{
		ip.setFancyGraphics(b);
		performanceFancyGraphics.setState(b);
	}

	/**
	 * Processes the MessageEvents generated.
	 * Implementation of the MessageListener interface.
	 * @param e MessageEvent for the event.
	 */
	public void messageReceived(MessageEvent e)
	{
		Object source = e.getSource();
		String action = e.getAction();
		String message = e.getMessage();
		
		// Message from DvaspiPanel
		if (source == ip)
		{
			if (action.equals("moved"))
			{
				statusBar.setLocation(message);
			}
			else if (action.equals("probe"))
			{
				statusBar.setMessage(message);
				dc.appendln("Probe: " + message);
			}
			else if (action.equals("select"))
			{
				statusBar.setMessage(message);
				dc.appendln(message);
			}
			else if (action.equals("closest"))
			{
				statusBar.setMessage(message);
				dc.appendln("Closest point is " + message);
			}
			else if (action.equals("integration"))
			{
				statusBar.setMessage(message);
				dc.appendln(message);
			}
		}
		// Message from DvapsiSelectionTable
		else if (source == dst)
		{
			if (action.equals("copy"))
			{
				dc.appendln("\nCopied following data points to the clipboard");
				dc.appendln("-----------------------------------------------");
				dc.appendln(message);
			}
			else if (action.equals("select"))
			{
				statusBar.setMessage(message);
				dc.appendln("Selection Table: " + message);
			}
			else if (action.equals("showerror"))
			{
				showErrorMessage(message, "Error");
			}
			else if (action.equals("drawonce"))
			{
				ip.drawOnce();
			}
		}
		// Message from DvaspiPropertiesPanel
		else if (source == dpp)
		{
			if (action.equals("error"))
			{
				showErrorMessage(message, "Error");
			}
			else if (action.equals("change"))
			{
				ip.drawOnce();
			}
			else if (action.equals("changecolor"))
			{
				dataSetList.repaint();
				ip.drawOnce();
			}
			else if (action.equals("rename"))
			{
				dataSetList.repaint();
			}
		}		
	}
	
	/**
	 * Processes the event when the dataSetList is changed.
	 * @param e ListSelectionEvent triggered from changing the
	 * value of the dataSetList.
	 */
	public void valueChanged(ListSelectionEvent e)
	{
		if (dataSetList.getSelectedIndices().length == 0)
			setDataSetButtonState(false);
		else
			setDataSetButtonState(true);
		
		if (dataSetList.getSelectedIndices().length < 2)
			dataSetMatchViewportsButton.setEnabled(false);
		else
			dataSetMatchViewportsButton.setEnabled(true);
		
		if (ip.getMode() != DvaspiPanel.MODE_NONE)
		{
			showErrorMessage("Cannot change data sets during processing.", "Error");
			
			// The tempmode workaround is to prevent the error dialog to
			// appear twice.
			// Calling setSelectedValue fires another valueChanged event,
			// and that leads to a second dialog. Workaround stops it.
			int tempmode = ip.getMode();
			
			ip.setMode(DvaspiPanel.MODE_NONE);
			dataSetList.setSelectedValue(ip.getIsolatedDS(), true);
			
			ip.setMode(tempmode);
			return;
		}
		
		DataSet ds = (DataSet)dataSetList.getSelectedValue();
		
		dst.updateSelectedTable(ds);
		dpp.updatePropertiesTable(ds);
	}

	/**
	 * 
	 * 
	 */
	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		
		if (debug)
			System.out.println(source);
		
		else if (source == dataSetRemoveButton)
		{
			processActionDataSetRemoveButton();
		}
		else if (source == dataSetFitToViewportButton)
		{
			processActionDataSetFitToViewportButton();
		}
		else if (source == dataSetMatchViewportsButton)
		{
			processActionDataSetMatchViewportsButton();
		}
		else if (source == fileOpenMenuItem)
		{
			processActionOpenFileMenuItem();
		}
		else if (source == fileExportMenuItem)
		{
			showSaveFileChooser(ip.getWidth(), ip.getHeight());
		}
		else if (source == fileExportSpecialMenuItem)
		{
			new ExportSpecialDialogBox(mainFrame, "Export Special", true);
		}
		else if (source == fileExitMenuItem)
		{
			mainFrame.dispose();
			System.exit(0);
		}
		else if (source == processIntegrateMenuItem)
		{
			ip.integrate();
		}
		else if (source == helpAboutMenuItem)
		{
			new AboutDialogBox(mainFrame, "About Coobird's Data Viewing and Selecting Program with Integration", true);
		}
		else if (source == modeNoneMenuItem || source == modeSelectMenuItem ||
					source == modeProbeMenuItem || source == modeClosestMenuItem ||
					source == modeIntegrateMenuItem || source == modeZoomInMenuItem ||
					source == modeZoomOutMenuItem || source == modeZoomCenterMenuItem)
		{
			processActionModeMenu(source);
		}
		else if (source == viewShowPointsMenuItem)
		{
			ip.setShowPoints(viewShowPointsMenuItem.isSelected());
			ip.drawOnce();
		}
		else if (source == viewShowLinesMenuItem)
		{
			ip.setShowLines(viewShowLinesMenuItem.isSelected());
			ip.drawOnce();
		}
		else if (source == viewColorBackgroundMenuItem)
		{
			Color colorPicked = JColorChooser.showDialog(mainFrame, "Background Color Selection", ip.getBackgroundColor());
			if (colorPicked != null)
				ip.setBackgroundColor(colorPicked);
			
			ip.drawOnce();
		}
		else if (source == viewDockPropertiesMenuItem)
		{
			if (propertiesDialog.isVisible())
			{
				propertiesDialog.setVisible(false);
				viewDockPropertiesMenuItem.setText("Undock Properties");
				propertiesDialog.getContentPane().remove(dpp);
				tp.add(dpp);
			}
			else
			{
				viewDockPropertiesMenuItem.setText("Dock Properties");
				propertiesDialog.getContentPane().add(dpp);
				propertiesDialog.setLocation(mainFrame.getX() + mainFrame.getWidth(), mainFrame.getY());
				propertiesDialog.setSize(200, mainFrame.getHeight());
				propertiesDialog.setVisible(true);
				propertiesDialog.validate();
				tp.remove(dpp);
			}
		}
		else if (source == performanceFancyGraphics)
		{
			ip.setFancyGraphics(performanceFancyGraphics.getState());
			ip.drawOnce();
		}
		else if (source == performanceFastGraphics)
		{
			ip.setFastGraphics(performanceFastGraphics.getState());
			ip.drawOnce();
		}
		else if (source == ip)
		{
			String action = e.getActionCommand();

			if (action.equals("select"))
			{
				dst.updateSelectedTable(ip.getIsolatedDS());
			}
			else if (action.equals("zoomevent"))
			{
				// TODO
				// Rewrite this to allow zoom of multiple
				// data sets at once
				
				DataSet ds = ip.getIsolatedDS();

				// Update Properties
				dpp.updatePropertiesTable(ds);
				dwp.repaint();
			}
		}
	}
	
	/**
	 * Dumps exception info to console
	 * Deprecated.
	 * @param e Exception.
	 */
	private void dumpExceptionInfo(Exception e)
	{
		dc.appendln("\n\n*** Exception ***");
		dc.appendln("An error has occurred:");
		dc.appendln(e.toString());
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);		
		e.printStackTrace(pw);
		dc.appendln(sw.toString());
		
		showErrorMessage("Exception: " + e.toString(), "Exception");
	}
	
	/**
	 * Main method.
	 * @param args Command line arguments.
	 */
	public static void main(String[] args)
	{
		try
		{
	    // Set System L&F
        UIManager.setLookAndFeel
        (
            UIManager.getSystemLookAndFeelClassName());
		} 
		catch (UnsupportedLookAndFeelException e)
		{
		   e.printStackTrace(System.out);
		}
		catch (ClassNotFoundException e)
		{
		   e.printStackTrace(System.out);
		}
		catch (InstantiationException e)
		{
		   e.printStackTrace(System.out);
		}
		catch (IllegalAccessException e)
		{
		   e.printStackTrace(System.out);
		}
		//Schedule a job for the event-dispatching thread:
        //creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                @SuppressWarnings("unused") DvaspiGui app = new DvaspiGui();
                app.makeGUI();
            }
        });
	}
	
	
	class CustomFileFilter extends javax.swing.filechooser.FileFilter
	{
		private HashSet<String> exts;
		private String description = "No description";
		private String type = "No type";
		
		public CustomFileFilter()
		{
			exts = new HashSet<String>();
		}
		
		public void addExtension(String ext)
		{
			exts.add(ext);
		}
		
		public boolean isExtension(String ext)
		{
			return exts.contains(ext);
		}

		public String getType()
		{
			return type;
		}
		
		public void setType(String type)
		{
			this.type = type;
		}
		
		public void setDescription(String desc)
		{
			description = desc;
		}
		public String getDescription()
		{
			return description;
		}
		
		private String getExtension(String filename)
		{
			int index = filename.lastIndexOf('.');
			
			if (index == -1)
				return null;
			
			return filename.substring(index+1);
		}
	
		public boolean accept(File f)
		{
			String ext = getExtension(f.getName().toLowerCase());

			if (exts.contains(ext) || f.isDirectory())
				return true;
			
			return false;
		}
	}
	
	class OpenFileThread implements Runnable
	{
		File f;
		DataSet ds;
		
		public OpenFileThread(File f, DataSet ds)
		{
			this.f = f;
			this.ds = ds;
		}
		
		public void run()
		{
	    	JDialog d = new JDialog(mainFrame, "Reading file...");
	    	JLabel l = new JLabel("Reading file... " + f.getName());
	    	
	    	d.setLayout(new FlowLayout());
	    	d.add(l);
	    	
	    	Point dLoc = mainFrame.getLocation();
	    	dLoc.translate(mainFrame.getWidth() / 2 - d.getWidth() / 2,
	    					mainFrame.getHeight() / 2 - d.getHeight() / 2);
	    	
	    	d.setLocation(dLoc);
	    	d.setVisible(true);
	    	d.setSize(l.getWidth() + 50, l.getHeight() + 50);
	    	d.validate();

	    	// Returns true if successful
	    	boolean success = Dvaspi.getData(f, ds);
	    	
	    	// If a problem arised, then throw a message and close dialog
	    	if (!success)
	    	{
	    		showErrorMessage("An error occurred when opening file: " + f.getName(), "Error opening file");
		    	d.setVisible(false);
		    	d.dispose();
	    		return;
	    	}
	    	
	    	// set the color for DataSet
	    	ds.lineColor = colorForLine.getNextElement();
	    	// add the DataSet to the DvaspiPanel
	    	ip.addDataSet(ds);
	    	// updates the DataSetPanel
	    	doListUpdate();
	    	
	    	if (ds.x.length >= LENGTH_MANYPOINTS && !ip.getFastGraphics())
	    	{
		    	int response = JOptionPane.showConfirmDialog(mainFrame, 
    					"It appears that the data set contains many points.\n" +
    					"Would you like to enable Fast Graphics mode?",
    					"Enable Fast Graphics", JOptionPane.YES_NO_OPTION,
    					JOptionPane.QUESTION_MESSAGE);
	    		
		    	if (response == JOptionPane.YES_OPTION)
		    		setFastGraphics(true);
		    	else
		    		setFastGraphics(false);
	    	}
	    	
	    	ip.drawOnce();
	    	dwp.repaint();

	    	d.setVisible(false);
	    	d.dispose();
		}		
	}
	
	@SuppressWarnings("serial")
	class AboutDialogBox extends JDialog
	{
		public AboutDialogBox(Frame f, String s, boolean b)
		{
			super(f, s, b);
			
			String msg =

				"Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)\n" +
				"Copyright (C) 2008 Chris Kroells\n\n" +
				
				"Dvaspi is free software: you can redistribute it and/or modify " +
				"it under the terms of the GNU General Public License as published by " +
				"the Free Software Foundation, either version 3 of the License, or " +
				"(at your option) any later version.\n\n" +
				
				"Dvapsi is distributed in the hope that it will be useful, " +
				"but WITHOUT ANY WARRANTY; without even the implied warranty of " +
				"MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the " +
				"GNU General Public License for more details.\n\n" +
				
				"You should have received a copy of the GNU General Public License " +
				"along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.";				
								
			JPanel p = new JPanel(new GridLayout(0, 1));
						
			p.add(new JLabel("Coobird's Data Viewing and Selecting Program with Integration (DVASPI)"));
			p.add(new JLabel("Version " + version));
			
			JTextArea t = new JTextArea(10, 40);
			t.setLineWrap(true);
			t.setWrapStyleWord(true);
			JScrollPane sp = new JScrollPane(t, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
			t.setFont(new Font("Monospaced", Font.PLAIN, 12));
			t.setText(msg);
			t.setCaretPosition(0);
			t.setEditable(false);
			
			getContentPane().setLayout(new BorderLayout());
			JButton okButton = new JButton("OK");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e)
				{
					setVisible(false);
					dispose();
				}				
			});
			getContentPane().add(p, BorderLayout.NORTH);
			getContentPane().add(okButton, BorderLayout.SOUTH);
			getContentPane().add(sp, BorderLayout.CENTER);
			pack();
			int x = mainFrame.getLocation().x + (mainFrame.getWidth() / 2) - (getWidth() / 2);
			int y = mainFrame.getLocation().y + (mainFrame.getHeight() / 2) - (getHeight() / 2);
			setLocation(x, y);
			setVisible(true);
		}
	}
	
	@SuppressWarnings("serial")
	class ExportSpecialDialogBox extends JDialog
	{
		public ExportSpecialDialogBox(Frame f, String s, boolean b)
		{
			super(f, s, b);
			
			setResizable(false);
			
			final JTextField widthTF = new JTextField();
			final JTextField heightTF = new JTextField();
			JPanel p = new JPanel(new GridLayout(0, 1));
			JPanel p2 = new JPanel(new GridLayout(2, 0));
			p2.add(new JLabel("Width:"));
			p2.add(widthTF);
			p2.add(new JLabel("Height:"));
			p2.add(heightTF);
			
			p.add(new JLabel("Enter dimensions for the output image:"));
			p.add(p2);
			
			getContentPane().setLayout(new BorderLayout());
			JButton okButton = new JButton("OK");
			okButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e)
				{
					try
					{
						int width = Integer.parseInt(widthTF.getText());
						int height = Integer.parseInt(heightTF.getText());
						setVisible(false);
						dispose();
						showSaveFileChooser(width, height);
					}
					catch (Exception ex)
					{
						showErrorMessage("Dimension must be a number.", "Invalid input", ex);
					}
				}				
			});
			JButton cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e)
				{
					setVisible(false);
					dispose();
				}				
			});
			
			JPanel p3 = new JPanel(new GridLayout(0, 2));
			p3.add(okButton);
			p3.add(cancelButton);
			getContentPane().add(p, BorderLayout.CENTER);
			getContentPane().add(p3, BorderLayout.SOUTH);
			pack();
			int x = mainFrame.getLocation().x + (mainFrame.getWidth() / 2) - (getWidth() / 2);
			int y = mainFrame.getLocation().y + (mainFrame.getHeight() / 2) - (getHeight() / 2);
			setLocation(x, y);
			setVisible(true);
		}
	}
	
	@SuppressWarnings("serial")
	class MatchViewportsDialog extends JDialog implements ActionListener
	{
		public static final int OK_BUTTON = 0;
		public static final int CANCEL_BUTTON = 1;
		
		public static final int FIT_TO_LARGEST = 0;
		public static final int FIT_TO_ALL = 1;
		public static final int FIT_TO_DATASET_ABS = 2;
		public static final int FIT_TO_DATASET = 3;
		
		public int response;
		public int type;
		public DataSet dataSetToFitTo;
		
		private JButton okButton;
		private JButton cancelButton;
		
		private JComboBox dsCB;
		private Vector<DataSet> dsv;
		
		private JRadioButton fitToLargest;
		private JRadioButton fitToAll;
		private JRadioButton fitToDatasetAbs;
		private JRadioButton fitToDataset;

		
		public MatchViewportsDialog(Frame f, String s, boolean b)
		{
			super(f, s, b);
			
			setResizable(false);
			
			JPanel mainPanel = new JPanel(new GridLayout(0, 1));
			dsCB = new JComboBox();
			
			dsv = ip.getDataSetVector();
			
			for (int i = 0; i < dsv.size(); i++)
			{
				dsCB.addItem(dsv.get(i).getName());
			}
			
			fitToLargest = new JRadioButton("Match viewports to data set with largest viewport", true);
			fitToAll = new JRadioButton("Match viewports to fit all data sets", true);
			fitToDatasetAbs = new JRadioButton("Match viewports to specified dataset's original viewport", false);
			fitToDataset = new JRadioButton("Match viewports to specified dataset's current viewport", false);
			fitToLargest.addActionListener(this);
			fitToAll.addActionListener(this);
			fitToDatasetAbs.addActionListener(this);
			fitToDataset.addActionListener(this);
			ButtonGroup fitButtons = new ButtonGroup();
			fitButtons.add(fitToLargest);
			fitButtons.add(fitToAll);
			fitButtons.add(fitToDatasetAbs);
			fitButtons.add(fitToDataset);
			dsCB.setEnabled(false);
			
			okButton = new JButton("OK");
			okButton.addActionListener(this);
			cancelButton = new JButton("Cancel");
			cancelButton.addActionListener(this);
			
			JPanel p3 = new JPanel(new GridLayout(0, 2));
			p3.add(okButton);
			p3.add(cancelButton);
			
			mainPanel.add(fitToLargest);
			mainPanel.add(fitToAll);
			mainPanel.add(fitToDatasetAbs);
			mainPanel.add(fitToDataset);
			mainPanel.add(dsCB);

			getContentPane().setLayout(new BorderLayout());
			getContentPane().add(mainPanel, BorderLayout.CENTER);
			getContentPane().add(p3, BorderLayout.SOUTH);
			pack();
			int x = mainFrame.getLocation().x + (mainFrame.getWidth() / 2) - (getWidth() / 2);
			int y = mainFrame.getLocation().y + (mainFrame.getHeight() / 2) - (getHeight() / 2);
			setLocation(x, y);
			setVisible(true);
		}
		
		public void actionPerformed(ActionEvent e)
		{
			Object source = e.getSource();
			
			if (source == okButton)
			{
				if (fitToLargest.isSelected())
					type = FIT_TO_LARGEST;
				else if (fitToDatasetAbs.isSelected())
					type = FIT_TO_DATASET_ABS;
				else if (fitToDataset.isSelected())
					type = FIT_TO_DATASET;
				else
					type = FIT_TO_ALL;
				
				dataSetToFitTo = dsv.elementAt(dsCB.getSelectedIndex());
				
				response = OK_BUTTON;
				setVisible(false);
				dispose();
				
			}
			else if (source == cancelButton)
			{
				response = CANCEL_BUTTON;
				setVisible(false);
				dispose();
			}
			else if (source == fitToLargest || source == fitToAll)
			{
				dsCB.setEnabled(false);
			}
			else if (source == fitToDataset || source == fitToDatasetAbs)
			{
				dsCB.setEnabled(true);
			}
		}
	}
}