//    Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)
//    Copyright (C) 2008 Chris Kroells
//
//    This file is part of Dvaspi.
//
//    Dvaspi is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Dvapsi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.


package net.coobird.dvaspi;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.text.DecimalFormat;
import java.util.*;
import javax.imageio.*;
import java.io.*;

/**
 * This class displays the data sets to the screen.  
 * @author coobird
 */
@SuppressWarnings("serial")
public class DvaspiPanel extends JPanel implements MouseListener, MouseMotionListener
{
	private Color backgroundColor = Color.white;
	private Image bufferImage;
	// Graphics of bufferImage
	private Graphics bg;
	private boolean fastLineDraw = true;
	private boolean fastPointDraw = true;
	private boolean fancyGraphics = true;
	
	private boolean showPoints = true;
	private boolean showLines = true;
	
	private DataSet isolatedDS;
	
	// Vector used to store the DataSets used in this DvaspiPanel
	private Vector<DataSet> dsVector;
	
	private int mode;
	public static int MODE_NONE = 0;
	public static int MODE_SELECT = 1;
	public static int MODE_PROBE = 2;
	public static int MODE_CLOSEST = 3;
	public static int MODE_INTEGRATE = 4;
	public static int MODE_ZOOM_IN = 5;
	public static int MODE_ZOOM_OUT = 6;
	public static int MODE_ZOOM_CENTER = 7;
	
	private Point lineStart;
	private Point lineEnd;
	
	private Color transparentZoomColor = new Color(0,0,128,128);
	
	private Vector<ActionListener> actionListeners;
	private Vector<MessageListener> messageListeners;
	private DecimalFormat formatting;
	
	/**
	 * Default constructor
	 *
	 */
	public DvaspiPanel()
	{
		dsVector = new Vector<DataSet>();
		actionListeners = new Vector<ActionListener>();
		messageListeners = new Vector<MessageListener>();
		formatting = new DecimalFormat("#.#####E0");
		
		this.addMouseListener(this);
		this.addMouseMotionListener(this);
	}
	
	/**
	 * Adds a DataSet object to display on the panel.
	 * @param ds DataSet object to add to the panel to display.
	 */
	public void addDataSet(DataSet ds)
	{
		dsVector.add(ds);
	}

	/**
	 * Removes a DataSet object that is being displayed on the panel.
	 * @param ds DataSet object to remove from the panel.
	 */
	public void removeDataSet(DataSet ds)
	{
		dsVector.remove(ds);
	}
	
	/**
	 * Returns a Vector object containing the DataSet objects to display on the panel.
	 * @return Vector object containing the DataSet objects to display on the panel.
	 */
	public Vector<DataSet> getDataSetVector()
	{
		return dsVector;
	}
	
	/**
	 * Sets the DataSet object currently being processed.
	 * @param ds DataSet to be processed.
	 */
	public void setIsolatedDS(DataSet ds)
	{
		isolatedDS = ds;
	}
	
	/**
	 * Returns the DataSet object currently being processed.
	 * @return DataSet object currently being processed.
	 */
	public DataSet getIsolatedDS()
	{
		return isolatedDS;
	}
	
	/**
	 * Clears the DataSet currently being processed.
	 */
	public void clearIsolatedDS()
	{
		isolatedDS = null;
	}
	
	/**
	 * Sets the processing mode.
	 * @param mode Processing mode (See DvaspiPanel mode constants)
	 */
	public void setMode(int mode)
	{
		this.mode = mode;
	}
	
	/**
	 * Gets the current processing mode for the DataSet
	 * @return Current processing mode. (See DvaspiPanel mode constants)
	 */
	public int getMode()
	{
		return mode;
	}
	
	/**
	 * Sets whether to use the fast graphics mode.
	 * Fast graphics mode will reduce the number of data points being
	 * displayed in the viewport.
	 * @param b A value of true will turn on fast graphics mode.
	 */
	public void setFastGraphics(boolean b)
	{
		fastLineDraw = b;
		fastPointDraw = b;
	}
	
	/**
	 * Returns whether or not fast graphics is enabled.
	 * @return Whether fast graphics mode is enabled.
	 */
	public boolean getFastGraphics()
	{
		return fastLineDraw && fastPointDraw;
	}
	
	/**
	 * Sets whether to use fancy graphics mode.
	 * Fancy graphics mode enables the use of transparency effects
	 * in processing mode.
	 * @param b A value of true will turn on fancy graphics mode.
	 */
	public void setFancyGraphics(boolean b)
	{
		fancyGraphics = b;
	}
	
	/**
	 * Returns whether or not fancy graphics is enabled.
	 * @return Whether fancy graphics mode is enabled.
	 */
	public boolean getFancyGraphics()
	{
		return fancyGraphics;
	}
	
	/**
	 * Sets whether to show points.
	 * @param b Displays points if true, not if false.
	 */
	public void setShowPoints(boolean b)
	{
		showPoints = b;
	}
	
	/**
	 * Sets whether to show lines.
	 * @param b Displays lines if true, not if false.
	 */
	public void setShowLines(boolean b)
	{
		showLines = b;
	}
	
	/**
	 * 
	 * @param l
	 */
	public void addActionListener(ActionListener l)
	{
		actionListeners.add(l);
	}
	
	/**
	 * 
	 * @param l
	 */
	public void removeActionListener(ActionListener l)
	{
		actionListeners.remove(l);
	}
	
	private void notifyActionListeners(String action)
	{
		for (Enumeration<ActionListener> e = actionListeners.elements(); e.hasMoreElements(); )
		{
			e.nextElement().actionPerformed(new ActionEvent(this, 0, action));
		}
	}

	public void addMessageListener(MessageListener l)
	{
		messageListeners.add(l);
	}
	
	public void removeMessageListener(MessageListener l)
	{
		messageListeners.remove(l);
	}
	
	private void notifyMessageListeners(String action, String msg)
	{
		for (Enumeration<MessageListener> e = messageListeners.elements(); e.hasMoreElements(); )
		{
			e.nextElement().messageReceived(new MessageEvent(this, action, msg));
		}
	}
	
	
	/**
	 * Calculates and scales the DataSet to the specified dimension
	 * @param ds DataSet to scale.
	 * @param w Width to scale to.
	 * @param h Height to scale to.
	 */
	private void calculate(DataSet ds, int w, int h)
	{		
		double width = ds.rightX - ds.leftX;
		// switch the minY and maxY if you want it upside down, and vice versa
		double height = ds.topY - ds.bottomY;
		double centerx = (ds.leftX + ds.rightX) / 2;
		double centery = (ds.topY + ds.bottomY) / 2;
		
		double xfactor = w / width;
		double yfactor = h / height;
		
		int offsetx = (int)Math.round(((width / 2) - centerx) * xfactor);
		int offsety = (int)Math.round(((height / 2) - centery) * yfactor);
		
		int[] sx = new int[ds.x.length];
		int[] sy = new int[ds.y.length];
		
		for (int i = 0; i < sx.length; i++)
		{
			sx[i] = (int)Math.round(ds.x[i] * xfactor) + offsetx;
			sy[i] = (int)Math.round(ds.y[i] * yfactor) + offsety;
		}
		
		ds.setScaledData(sx, sy);
		ds.setFactors(xfactor, yfactor, offsetx, offsety);
	}
	
	/**
	 * Draws the DataSets on the Panel
	 *
	 */
	public void drawOnce()
	{
		Graphics g = this.getGraphics();
		bg.dispose();
		bufferImage = this.createImage(this.getWidth(), this.getHeight());	
		bg = bufferImage.getGraphics();
		drawViewArea(bg, this.getWidth(), this.getHeight());
		g.drawImage(bufferImage, 0, 0, null);
		g.dispose();
	}
	
	/**
	 * Exports the current panel to file
	 * @param type Export file format
	 * @param f Output file
	 * @param width Width of the exported image
	 * @param height Height of the exported image
	 */
	public void exportImage(String type, File f, int width, int height)
	{
		try
		{
			BufferedImage img = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
			Graphics g = img.getGraphics();
			
			drawViewArea(g, width, height);
			ImageIO.write(img, type, f);
		}
		catch (Exception e)
		{
			e.printStackTrace(System.out);
		}
	}
	
	/**
	 * Paints the panel. 
	 * 
	 */
	public void paintComponent(Graphics g)
	{
		// if the bufferImage doens't exist yet, make one
		if (bufferImage == null)
		{
			bufferImage = this.createImage(this.getWidth(), this.getHeight());	
			bg = bufferImage.getGraphics();
		}
		
		// if the panel size changes
		if ( !( bufferImage.getWidth(null) == this.getWidth() &&
				bufferImage.getHeight(null) == this.getHeight() ) )
		{
			bg.dispose();
			bufferImage = this.createImage(this.getWidth(), this.getHeight());	
			bg = bufferImage.getGraphics();

			// draws the image according to the new panel size
			drawViewArea(bg, this.getWidth(), this.getHeight());
		}
		
		// draws the image to panel
		g.drawImage(bufferImage, 0, 0, null);
	}
	
	/**
	 * Draws one DataSet onto a specified Graphics object.
	 * @param ds DataSet object to draw.
	 * @param g Graphics object to draw to the DataSet object to.
	 * @param width Width of the display field.
	 * @param height Height of the display field.
	 */
	private void drawDataSet(DataSet ds, Graphics g, int width, int height)
	{
		// do calculations to determine screen position of data points
		calculate(ds, width, height);
		
		// stop here, there's nothing to show
		if (ds.sx == null)
			return;

		// set default pen color
		g.setColor(Color.black);
		
		// find out the range of points that we need to draw
		// this will speed up the drawing speed when drawing portion of the DataSet
		int startPoint = 0;
		int endPoint = ds.sx.length;

		for (endPoint = 0; endPoint < ds.sx.length; endPoint++)
			if (ds.x[endPoint] > ds.rightX)
				break;

		if (endPoint > 0)	// trust me, uncaught exception later if endpoint = 0
			for (startPoint = endPoint - 1; startPoint > 0; startPoint--)
				if (ds.x[startPoint] < ds.leftX)
					break;
				
		
		if (ds.drawLines && showLines)
		{
			g.setColor(ds.lineColor);
			
			int numPoints = endPoint - startPoint;
			int cnt = 1;
			
			if (fastLineDraw)
				cnt = (numPoints / width) + 1;

			for (int i = startPoint; i < endPoint - cnt; i = i + cnt)
			{
				g.drawLine(ds.sx[i], ds.sy[i], ds.sx[i + cnt], ds.sy[i + cnt]);
			}			
		}

		if (ds.drawPoints && showPoints)
		{
			int radius = ds.getPointRadius();
			int diameter = ds.getPointDiameter();
			Color colorSelected = ds.pointSelectedColor;
			Color colorPoint = ds.pointColor;
			
			if (fastPointDraw)
			{
				int numPoints = endPoint - startPoint;
				int cnt = (numPoints / width) + 1;

				for (int i = startPoint; i < endPoint - cnt; i = i + cnt)
				{
					if (ds.selected[i])
					{
						g.setColor(colorSelected);
						g.drawOval(ds.sx[i] - radius, ds.sy[i]- radius, diameter, diameter);
					}
					else
					{
						g.setColor(colorPoint);
						g.drawOval(ds.sx[i] - radius, ds.sy[i]- radius, diameter, diameter);
					}
				}				
			}
			else
			{
				for (int i = startPoint; i < endPoint; i++)
				{
					if (ds.selected[i])
					{
						g.setColor(colorSelected);
						g.drawOval(ds.sx[i] - radius, ds.sy[i]- radius, diameter, diameter);
					}
					else
					{
						g.setColor(colorPoint);
						g.drawOval(ds.sx[i] - radius, ds.sy[i]- radius, diameter, diameter);
					}
				}
			}
		}
		
		if (ds.getLabelMode() != DataSet.MODE_SHOW_NONE && showPoints)
		{
			int labelMode = ds.getLabelMode();
			
			g.setColor(ds.labelColor);
			
			if (fastPointDraw)
			{
				int numPoints = endPoint - startPoint;
				int cnt = (numPoints / width) + 1;

				if (ds.drawXLabel && ds.drawYLabel)
				{
					for (int i = startPoint; i < endPoint - cnt; i = i + cnt)
						if ((ds.selected[i] && labelMode == DataSet.MODE_SHOW_SELECTED) || (labelMode == DataSet.MODE_SHOW_ALL))
							g.drawString("" + ds.x[i] + ", " + ds.y[i], ds.sx[i]+10, ds.sy[i]+10);
				}
				else if (ds.drawXLabel)
				{
					for (int i = startPoint; i < endPoint - cnt; i = i + cnt)
						if ((ds.selected[i] && labelMode == DataSet.MODE_SHOW_SELECTED) || (labelMode == DataSet.MODE_SHOW_ALL))
							g.drawString("" + ds.x[i], ds.sx[i]+10, ds.sy[i]+10);
				}
				else if (ds.drawYLabel)
				{
					for (int i = startPoint; i < endPoint - cnt; i = i + cnt)
						if ((ds.selected[i] && labelMode == DataSet.MODE_SHOW_SELECTED) || (labelMode == DataSet.MODE_SHOW_ALL))
							g.drawString("" + ds.y[i], ds.sx[i]+10, ds.sy[i]+10);
				}
			}
			else
			{
				if (ds.drawXLabel && ds.drawYLabel)
				{
					for (int i = startPoint; i < endPoint; i++)
						if ((ds.selected[i] && labelMode == DataSet.MODE_SHOW_SELECTED) || (labelMode == DataSet.MODE_SHOW_ALL))
							g.drawString("" + ds.x[i] + ", " + ds.y[i], ds.sx[i]+10, ds.sy[i]+10);
				}
				else if (ds.drawXLabel)
				{
					for (int i = startPoint; i < endPoint; i++)
						if ((ds.selected[i] && labelMode == DataSet.MODE_SHOW_SELECTED) || (labelMode == DataSet.MODE_SHOW_ALL))
							g.drawString("" + ds.x[i], ds.sx[i]+10, ds.sy[i]+10);
				}
				else if (ds.drawYLabel)
				{
					for (int i = startPoint; i < endPoint; i++)
						if ((ds.selected[i] && labelMode == DataSet.MODE_SHOW_SELECTED) || (labelMode == DataSet.MODE_SHOW_ALL))
							g.drawString("" + ds.y[i], ds.sx[i]+10, ds.sy[i]+10);
				}
			}
		}
	}
	
	/**
	 * Draws the view area.
	 * @param g Graphics object to draw to.
	 * @param width Width of the view area.
	 * @param height Height of the view area.
	 */
	private void drawViewArea(Graphics g, int width, int height)
	{
		// clear the screen
		g.setColor(backgroundColor);
		g.fillRect(0, 0, width, height);
		
		if (isolatedDS == null)
		{
			for (Enumeration<DataSet> dsEnum = dsVector.elements(); dsEnum.hasMoreElements(); )
				drawDataSet(dsEnum.nextElement(), g, width, height);
		}
		else
		{
			// Fast draw settings are disabled during isolation/processing mode
			// Using temporary variables to hold the fast draw settings temporarily.
			boolean tempFastLineDraw = fastLineDraw;
			boolean tempFastPointDraw = fastPointDraw;
			fastLineDraw = false;
			fastPointDraw = false;

			// The isolated DataSet is drawn normally in solid color
			drawDataSet(isolatedDS, g, width, height);
			
			// When Fancy Graphics is enabled, other lines are drawn
			// with some transparency.
			if (fancyGraphics)
			{
				for (Enumeration<DataSet> dsEnum = dsVector.elements(); dsEnum.hasMoreElements(); )
				{
					DataSet ds = dsEnum.nextElement();
					
					// Creates the new Colors to use when drawing with alpha
					if (ds != isolatedDS)
					{
						Color c = ds.lineColor;
						ds.lineColor = new Color(c.getRed(), c.getGreen(), c.getBlue(), 64);
						c = ds.pointColor;
						ds.pointColor = new Color(c.getRed(), c.getGreen(), c.getBlue(), 64);
						c = ds.pointSelectedColor;
						ds.pointSelectedColor = new Color(c.getRed(), c.getGreen(), c.getBlue(), 64);
						c = ds.labelColor;
						ds.labelColor = new Color(c.getRed(), c.getGreen(), c.getBlue(), 64);
						
						drawDataSet(ds, g, width, height);
						
						c = ds.lineColor;
						ds.lineColor = new Color(c.getRed(), c.getGreen(), c.getBlue());
						c = ds.pointColor;
						ds.pointColor = new Color(c.getRed(), c.getGreen(), c.getBlue());
						c = ds.pointSelectedColor;
						ds.pointSelectedColor = new Color(c.getRed(), c.getGreen(), c.getBlue());
						c = ds.labelColor;
						ds.labelColor = new Color(c.getRed(), c.getGreen(), c.getBlue());
					}
				}
			}
			
			if (mode == MODE_INTEGRATE)
			{
				// While in Integrate mode, show the integration baseline
				if (lineStart != null && lineEnd != null)
				{
					g.setColor(Color.black);
					g.drawLine(lineStart.x, lineStart.y, lineEnd.x, lineEnd.y);
				}
			}
			else if (mode == MODE_ZOOM_IN)
			{
				// While in zoom in mode, draw the zoom selection
				if (lineStart != null && lineEnd != null)
				{
					if (fancyGraphics)
					{
						g.setColor(transparentZoomColor);
						g.fillRect(lineStart.x, lineStart.y, lineEnd.x - lineStart.x, lineEnd.y - lineStart.y);
					}
					else
					{
						g.drawRect(lineStart.x, lineStart.y, lineEnd.x - lineStart.x, lineEnd.y - lineStart.y);
					}
				}
				g.setColor(Color.black);
			}

			
			// Restore pre-modified fast draw settings
			fastLineDraw = tempFastLineDraw;
			fastPointDraw = tempFastPointDraw;
		}
	}
	
	/**
	 * Returns the background color of the Panel.
	 * @return Color object representing the background color of the panel.
	 */
	public Color getBackgroundColor()
	{
		return backgroundColor;
	}
	
	/**
	 * Sets the background color of the Panel.
	 * @param c Background color to set to.
	 */
	public void setBackgroundColor(Color c)
	{
		backgroundColor = c;
	}
	
	/**
	 * Calculates the distance between two points
	 * @param sx The x-coordinate for the first point.
	 * @param sy The y-coordinate for the first point.
	 * @param x The x-coordinate for the second point.
	 * @param y The y-coordinate for the second point.
	 * @return Distance between two points.
	 */
	private double dist(double sx, double sy, int x, int y)
	{
		double distance = Math.sqrt(Math.pow(sx-x,2)+Math.pow(sy-y,2));
		return distance;
	}
	
	/**
	 * Find the closest point in the DataSet based on the mouse click position
	 * on the panel.
	 * @param x Mouse x position.
	 * @param y Mouse y position.
	 * @return Index of the closest point in the DataSet
	 */
	private int findClosestIndex(int x, int y)
	{
		double minDist = (double)this.getWidth();
		int minDistIndex = 0;
		
		// Find out the range of points that we need to draw
		// This will speed up the selection speed due to less points havings its
		// distance calculated
		int startPoint = 0;
		int endPoint = isolatedDS.sx.length;
		
		for (endPoint = 0; endPoint < isolatedDS.sx.length; endPoint++)
			if (isolatedDS.x[endPoint] > isolatedDS.rightX)
				break;

		for (startPoint = endPoint - 1; startPoint > 0; startPoint--)
			if (isolatedDS.x[startPoint] < isolatedDS.leftX)
				break;
		
		// Find the closest point
		for (int i = startPoint; i < endPoint; i++)
		{
			if (dist(isolatedDS.sx[i], isolatedDS.sy[i], x, y) < minDist)
			{
				minDist = dist(isolatedDS.sx[i], isolatedDS.sy[i], x, y);
				minDistIndex = i;
			}
		}

		return minDistIndex;
	}
	
	/**
	 * Performs integration.
	 */
	public void integrate()
	{
		if (lineStart == null || lineEnd == null)
		{
			System.out.println("Cannot integrate without start and end points defined.");
			return;
		}
		
		double startx = (lineStart.x - isolatedDS.offsetx) / isolatedDS.xfactor;
		double starty = (lineStart.y - isolatedDS.offsety) / isolatedDS.yfactor;
		double endx = (lineEnd.x - isolatedDS.offsetx) / isolatedDS.xfactor;
		double endy = (lineEnd.y - isolatedDS.offsety) / isolatedDS.yfactor;
		
		if (endx == startx)
		{
			System.out.println("Cannot integrate vertically.");
			return;
		}

		// slope of the cut line
		double slope = (endy - starty) / (endx - startx);
		
		// find index of point coming after the starting point
		int startOfInterval=0;
		for (int i = 0; i < isolatedDS.sx.length; i++)
		{
			if (isolatedDS.x[i] > startx)
			{
				startOfInterval = i;
				break;
			}
		}

		int endOfInterval=0;
		for (int i = 0; i < isolatedDS.sx.length; i++)
		{
			if (isolatedDS.x[i] > endx)
			{
				endOfInterval = i;
				break;
			}
		}
				
		try
		{
			double area = 0;
			double diff0 = 0;
			double diff1 = 0;
			double slice = 0;
			double yOnLine = starty;
			for (int i = startOfInterval+1; i < endOfInterval; i++)
			{
				yOnLine = slope * (isolatedDS.x[i] - isolatedDS.x[i-1]) + yOnLine;
				diff0 = Math.abs(yOnLine - isolatedDS.y[i]);
				yOnLine = slope * (isolatedDS.x[i+1] - isolatedDS.x[i]) + yOnLine;
				diff1 = Math.abs(yOnLine - isolatedDS.y[i+1]);
				
				// trapezoid rule
				slice = (isolatedDS.x[i+1] - isolatedDS.x[i]) * ((diff0 + diff1) / 2);
				area += slice;
			}
			notifyActionListeners("integration");
			notifyMessageListeners("integration", "\nIntegrated from indices " + startOfInterval + " to " + endOfInterval);
			notifyMessageListeners("integration", "Area: " + area);
		}
		catch (Exception e)
		{
			System.out.println(e.toString());
		}
	}

	/**
	 * Processes a mouse clicked event.
	 * @param e MouseEvent object for the event.
	 */
	public void mouseClicked(MouseEvent e)
	{
		if (isolatedDS != null)
		{
			int x = e.getX();
			int y = e.getY();
			
			if (mode == MODE_SELECT)
			{
				int i = findClosestIndex(x,y);
				isolatedDS.selected[i] = !isolatedDS.selected[i];
				notifyActionListeners("select");
				
				if (isolatedDS.selected[i])
					notifyMessageListeners("select", "Selected point at index " + i);
				else
					notifyMessageListeners("select", "Deselected point at index " + i);
				
				drawOnce();
			}
			else if (mode == MODE_PROBE)
			{
				/*
				double ratioX = x / (double)this.getWidth();
				double ratioY = y / (double)this.getHeight();
				System.out.println("x: " + x + "\ty: " + y);
				
				double actualX = (Math.abs(isolatedDS.rightX - isolatedDS.leftX) * ratioX) + isolatedDS.leftX;
				double actualY = -((Math.abs(isolatedDS.topY - isolatedDS.bottomY) * ratioY) - isolatedDS.bottomY);				
				
				System.out.println("x: " + actualX + "\ty: " + actualY);
				*/
				
				double locx = (e.getX() - isolatedDS.offsetx) / isolatedDS.xfactor;
				double locy = (e.getY() - isolatedDS.offsety) / isolatedDS.yfactor;
				
				notifyMessageListeners("probe", formatXY(locx, locy));
			}
			else if (mode == MODE_CLOSEST)
			{
				int index = findClosestIndex(x, y);
				notifyMessageListeners("closest", formatXY(isolatedDS.x[index], isolatedDS.y[index]));
			}
		}		
	}

	
	/**
	 * Processes the mouse pressed event.
	 * @param e MouseEvent object for event. 
	 */
	public void mousePressed(MouseEvent e)
	{
		if (mode == MODE_INTEGRATE || mode == MODE_ZOOM_IN)
		{
			lineStart = new Point(e.getX(), e.getY());
		}
	}

	/**
	 * Processes the mouse released event.
	 * @param e MouseEvent object for the event.
	 */
	public void mouseReleased(MouseEvent e)
	{
		if (mode == MODE_INTEGRATE)
		{
			lineEnd = new Point(e.getX(), e.getY());
			
			if (lineEnd.equals(lineStart))
			{
				lineEnd = null;
				lineStart = null;
			}
			
			drawOnce();
		}
		else if (mode == MODE_ZOOM_IN)
		{
			lineEnd = new Point(e.getX(), e.getY());
			
			if (lineEnd.equals(lineStart))
			{
				lineEnd = null;
				lineStart = null;
				return;
			}

			isolatedDS.leftX = (lineStart.x - isolatedDS.offsetx) / isolatedDS.xfactor;;
			isolatedDS.rightX = (lineEnd.x - isolatedDS.offsetx) / isolatedDS.xfactor;;
			isolatedDS.topY = (lineEnd.y - isolatedDS.offsety) / isolatedDS.yfactor;
			isolatedDS.bottomY = (lineStart.y - isolatedDS.offsety) / isolatedDS.yfactor;
			
			lineStart = null;
			lineEnd = null;

			notifyActionListeners("zoomevent");
			drawOnce();
		}
		else if (mode == MODE_ZOOM_OUT || mode == MODE_ZOOM_CENTER)
		{
			double zoomFactor = 1;
			
			if (mode == MODE_ZOOM_OUT)
				zoomFactor = 1.1;
			
			double curWidth = isolatedDS.rightX - isolatedDS.leftX;
			double curHeight = isolatedDS.bottomY - isolatedDS.topY;
			
			double newCenterX = (e.getX() - isolatedDS.offsetx) / isolatedDS.xfactor;
			double newCenterY = (e.getY() - isolatedDS.offsety) / isolatedDS.yfactor;
			
			isolatedDS.leftX = newCenterX - ((curWidth / 2) * zoomFactor);
			isolatedDS.rightX = newCenterX + ((curWidth / 2) * zoomFactor);
			isolatedDS.topY = newCenterY - ((curHeight / 2) * zoomFactor);
			isolatedDS.bottomY = newCenterY + ((curHeight / 2)* zoomFactor);
		
			notifyActionListeners("zoomevent");
			drawOnce();
		}
	}

	public void mouseEntered(MouseEvent e)
	{
	}

	public void mouseExited(MouseEvent e)
	{
	}

	public void mouseDragged(MouseEvent e)
	{
		if (mode == MODE_INTEGRATE || mode == MODE_ZOOM_IN)
		{
			lineEnd = new Point(e.getX(), e.getY());
			notifyOfMove(e);
			drawOnce();
		}
	}

	public void mouseMoved(MouseEvent e)
	{
		if (mode != MODE_NONE)
		{
			notifyOfMove(e);
		}
	}
	
	/**
	 * Triggers a MessageEvent and notifies the MessageListeners
	 * of the new position of the mouse.
	 * @param e
	 */
	private void notifyOfMove(MouseEvent e)
	{
		double locx = (e.getX() - isolatedDS.offsetx) / isolatedDS.xfactor;
		double locy = (e.getY() - isolatedDS.offsety) / isolatedDS.yfactor;
		
		String x = formatting.format(locx);
		String y = formatting.format(locy);

		notifyMessageListeners("moved", "" + x + ", " + y);
	}
	
	/**
	 * Formats the x- and y-coordinates.
	 * @param x The x-coordinate.
	 * @param y The y-coordinate.
	 * @return The formatted string with the x- and y-
	 * coordinates in the "x: <x value>, y: <y value>" format.
	 */
	private String formatXY(double x, double y)
	{
		String sx = formatting.format(x);
		String sy = formatting.format(y);
		
		return "x: " + sx + " y: " + sy;
	}
}