//    Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)
//    Copyright (C) 2008 Chris Kroells
//
//    This file is part of Dvaspi.
//
//    Dvaspi is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Dvapsi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.


package net.coobird.dvaspi;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.EmptyBorder;
import javax.swing.event.*;

// TODO
// 1. Allow use of keyboard to navigate and edit cells 

/**
 * Provides a table which can be used to view and edit the properties of
 * a given data set.
 * @author coobird
 */
@SuppressWarnings("serial")
public class DvaspiPropertiesPanel extends JPanel implements TableModelListener
{
	private JTable propTable;
	private JScrollPane propTableScrollPane;
	private PropertiesTableModel propTableModel;
	private PropertiesTableCellRenderer propTableCellRenderer;
	private PropertiesTableCellEditor propTableCellEditor;
	
	private DataSet selectedDS;
	
	private static final int ROW_NAME = 0;
	private static final int ROW_FILENAME = 1;
	private static final int ROW_LEFT = 2;
	private static final int ROW_RIGHT = 3;
	private static final int ROW_TOP = 4;
	private static final int ROW_BOTTOM = 5;
	private static final int ROW_DRAWPOINTS = 6;
	private static final int ROW_DRAWLINES = 7;
	private static final int ROW_LABELMODE = 8;
	private static final int ROW_DRAWXLABEL = 9;
	private static final int ROW_DRAWYLABEL = 10;
	private static final int ROW_LINECOLOR = 11;
	private static final int ROW_POINTCOLOR = 12;
	private static final int ROW_SELECTEDPOINTCOLOR = 13;
	private static final int ROW_LABELCOLOR = 14;
	
	private Vector<MessageListener> messageListeners;
	
	public DvaspiPropertiesPanel()
	{
		super(new BorderLayout());
		propTable = new JTable();
		propTableModel = new PropertiesTableModel();
		
		// Prepares the Properties table
		propTableModel.setColumnIdentifiers(new String[] {"Property", "Value"});
		propTableModel.addRow(new String[] {"Name", "0"});
		propTableModel.addRow(new String[] {"Filename", "0"});
		propTableModel.addRow(new String[] {"Left", "0"});
		propTableModel.addRow(new String[] {"Right", "0"});
		propTableModel.addRow(new String[] {"Top", "0"});
		propTableModel.addRow(new String[] {"Bottom", "0"});
		propTableModel.addRow(getBooleanRow("Draw Points"));
		propTableModel.addRow(getBooleanRow("Draw Lines"));
		propTableModel.addRow(getCharacterRow("Draw Labels"));
		propTableModel.addRow(getBooleanRow("Draw X Label"));
		propTableModel.addRow(getBooleanRow("Draw Y Label"));
		propTableModel.addRow(getColorRow("Line Color", Color.black));
		propTableModel.addRow(getColorRow("Point Color", Color.black));
		propTableModel.addRow(getColorRow("Selected Point Color", Color.black));
		propTableModel.addRow(getColorRow("Label Color", Color.black));
		propTableModel.addTableModelListener(this);
		propTable.setModel(propTableModel);
		propTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		
		// Sets the editor and renderer for the cells
		propTableCellRenderer = new PropertiesTableCellRenderer();
		propTableCellEditor = new PropertiesTableCellEditor();
		propTable.getColumnModel().getColumn(1).setCellRenderer(propTableCellRenderer);
		propTable.getColumnModel().getColumn(1).setCellEditor(propTableCellEditor);
		
		propTableScrollPane = new JScrollPane(propTable);
		this.add(propTableScrollPane, BorderLayout.CENTER);
		
		this.setName("Properties");
		
		// List of MessageListeners
		messageListeners = new Vector<MessageListener>();

		// TODO
		// Allow use of keyboard for editing.
		//propTable.addKeyListener(this);
		
		updatePropertiesTable(null);
	}
	
	/**
	 * Creates a table row that contains a Color
	 * @param s String to use for the "Property" column.
	 * @param c The Color to store.
	 * @return Returns an Object array to be added to the table.
	 */
	private Object[] getColorRow(String s, Color c)
	{
		Object[] o = new Object[2];
		o[0] = s;
		o[1] = c;
		
		return o;
	}
	
	/**
	 * Creates a table row that contains a Boolean.
	 * @param s String to use for the "Property" column.
	 * @return Returns an Object array to be added to the table.
	 */
	private Object[] getBooleanRow(String s)
	{
		Boolean b = new Boolean(true);
		Object[] o = new Object[2];
		o[0] = s;
		o[1] = b;
		
		return o;
	}
	
	/**
	 * Creates a table row that contains a Character.
	 * @param s String to use for the "Property" column.
	 * @return Returns an Object array to be added to the table.
	 */
	private Object[] getCharacterRow(String s)
	{
		Character c = new Character('A');
		Object[] o = new Object[2];
		o[0] = s;
		o[1] = c;
		
		return o;
	}

	/**
	 * Updates the Properties table
	 * @param ds The DataSet for which the properties should be displayed.
	 */
	public void updatePropertiesTable(DataSet ds)
	{
		if (ds == null)
		{
			this.removeAll();
			this.add(new JLabel("No data set selected."), BorderLayout.PAGE_START);
			this.validate();
			this.repaint();
			return;
		}
		else
		{
			if (!this.isAncestorOf(propTableScrollPane))
			{
				this.removeAll();
				this.add(propTableScrollPane, BorderLayout.CENTER);
				this.validate();
				this.repaint();
			}
		}
		
		selectedDS = ds;
		
		// rational for the remove and add is to prevent
		// sending "change" MessageEvents back to the GUI
		// each "change" event triggers a ip.drawOnce()
		// which seriously impacts performance
		propTableModel.removeTableModelListener(this);

		propTableModel.setValueAt(ds.getName(), ROW_NAME, 1);
		propTableModel.setValueAt(ds.getFilename(), ROW_FILENAME, 1);
		propTableModel.setValueAt(Double.toString(ds.leftX), ROW_LEFT, 1);
		propTableModel.setValueAt(Double.toString(ds.rightX), ROW_RIGHT, 1);
		propTableModel.setValueAt(Double.toString(ds.topY), ROW_TOP, 1);
		propTableModel.setValueAt(Double.toString(ds.bottomY), ROW_BOTTOM, 1);

		int mode = ds.getLabelMode();
		Character c = 'X';
		if (mode == DataSet.MODE_SHOW_ALL)
			c = new Character('A');
		else if (mode == DataSet.MODE_SHOW_NONE)
			c = new Character('N');
		else if (mode == DataSet.MODE_SHOW_SELECTED)
			c = new Character('S');
		propTableModel.setValueAt(c, ROW_LABELMODE, 1);
		
		propTableModel.setValueAt(new Boolean(ds.drawLines), ROW_DRAWLINES, 1);
		propTableModel.setValueAt(new Boolean(ds.drawPoints), ROW_DRAWPOINTS, 1);
		propTableModel.setValueAt(ds.lineColor, ROW_LINECOLOR, 1);
		propTableModel.setValueAt(ds.pointColor, ROW_POINTCOLOR, 1);
		propTableModel.setValueAt(ds.pointSelectedColor, ROW_SELECTEDPOINTCOLOR, 1);
		propTableModel.setValueAt(ds.labelColor, ROW_LABELCOLOR, 1);

		propTableModel.addTableModelListener(this);
	}
	
	/**
	 * Sets the DataSet object currently being processed.
	 * @param ds DataSet to be processed.
	 */
	public void setIsolatedDS(DataSet ds)
	{
		selectedDS = ds;
	}
	
	/**
	 * Returns the DataSet object currently being processed.
	 * @return DataSet object currently being processed.
	 */
	public DataSet getIsolatedDS()
	{
		return selectedDS;
	}
	
	/**
	 * Clears the DataSet currently being processed.
	 */
	public void clearIsolatedDS()
	{
		selectedDS = null;
	}

	/**
	 * Adds a MessageListener.
	 * @param l MessageListener to add.
	 */
	public void addMessageListener(MessageListener l)
	{
		messageListeners.add(l);
	}
	
	/**
	 * Removes a MessageListener.
	 * @param l MessageListener to remove.
	 */
	public void removeMessageListener(MessageListener l)
	{
		messageListeners.remove(l);
	}
	
	/**
	 * Notifies the registered MessageListeners of an event.
	 * This method prepares an MessageEvent object which is sent to
	 * all registered Message Listeners.
	 * @param action Action that is performed.
	 * @param msg Message that accompanies the action.
	 */
	private void notifyMessageListeners(String action, String msg)
	{
		for (Enumeration<MessageListener> e = messageListeners.elements(); e.hasMoreElements(); )
			e.nextElement().messageReceived(new MessageEvent(this, action, msg));
	}

	
	@SuppressWarnings("serial")
	class PropertiesTableModel extends DefaultTableModel
	{
		public boolean isCellEditable(int row, int col)
		{
			// Cells in first column is non-editable
			if (col == 0)
				return false;
			else if (row == 1)
				return false;
			else
				return true;
		}
	}
	
	/**
	 * Method checks to see if a number can be parsed as a double.
	 * @param s A String to parse.
	 * @return Returns true if a number can be parsed, false otherwise.
	 */
	private boolean canParse(String s)
	{
		try
		{
			// Try to see if string is parsable
			Double.parseDouble(s);
		}
		catch (NumberFormatException e)
		{
			// When a dimension that is not a valid double is entered,
			// NumberFormatException is thrown.
			notifyMessageListeners("error", "A dimension must be a number\n" + e.toString());
			return false;
		}
		catch (Exception e)
		{
			// Any other exception leads here.
			notifyMessageListeners("error", "An error occured.\n" + e.toString());
			return false;
		}
		
		// String can be parsed
		return true;
	}
	
	/**
	 * Returns the String stored at the (row, col) position of the table.
	 * @param row Row of the cell.
	 * @param col Column of the cell.
	 * @return String that is contained at the (row, col) position.
	 */
	private String getStringAt(int row, int col)
	{
		return (String)propTable.getValueAt(row, col);
	}
	
	/**
	 * Method called when the table is altered.
	 * @param e TableModelEvent of the event.
	 */
	public void tableChanged(TableModelEvent e)
	{
		Object source = e.getSource();
		
		if (source == propTableModel)
		{
			int row = e.getFirstRow();
			int col = e.getColumn();
			Object cellObj = propTableModel.getValueAt(row, col);
			
			if (cellObj instanceof Boolean)
			{
				if (row == ROW_DRAWPOINTS)
					selectedDS.drawPoints = ((Boolean)cellObj).booleanValue();
				else if (row == ROW_DRAWLINES)
					selectedDS.drawLines = ((Boolean)cellObj).booleanValue();
				if (row == ROW_DRAWXLABEL)
					selectedDS.drawXLabel = ((Boolean)cellObj).booleanValue();
				else if (row == ROW_DRAWYLABEL)
					selectedDS.drawYLabel = ((Boolean)cellObj).booleanValue();
				
				notifyMessageListeners("change", "");
			}
			else if (cellObj instanceof Color)
			{
				if (row == ROW_LINECOLOR)
					selectedDS.lineColor = (Color)cellObj;
				else if (row == ROW_POINTCOLOR)
					selectedDS.pointColor = (Color)cellObj;
				else if (row == ROW_SELECTEDPOINTCOLOR)
					selectedDS.pointSelectedColor = (Color)cellObj;
				else if (row == ROW_LABELCOLOR)
					selectedDS.labelColor = (Color)cellObj;
				
				notifyMessageListeners("changecolor", "");
			}
			else if (cellObj instanceof Character)
			{
				if (row == ROW_LABELMODE)
				{
					char c = ((Character)cellObj).charValue();
					
					if (c == 'A')
						selectedDS.setLabelMode(DataSet.MODE_SHOW_ALL);
					else if (c == 'S')
						selectedDS.setLabelMode(DataSet.MODE_SHOW_SELECTED);
					else if (c == 'N')
						selectedDS.setLabelMode(DataSet.MODE_SHOW_NONE);
					
					notifyMessageListeners("change", "");
				}
			}
			else if (cellObj instanceof String)
			{	
				String str = getStringAt(row, col);
				
				if (row == ROW_NAME)
				{
					selectedDS.setName(str);
					notifyMessageListeners("rename", "");
				}
				else if (row == ROW_LEFT || row == ROW_RIGHT ||
							row == ROW_TOP || row == ROW_BOTTOM)
				{
					if (canParse(str))
					{
						// set the data set
						if (row == ROW_LEFT)
							selectedDS.leftX = Double.parseDouble(str);
						else if (row == ROW_RIGHT)
							selectedDS.rightX = Double.parseDouble(str);
						else if (row == ROW_TOP)
							selectedDS.topY = Double.parseDouble(str);
						else if (row == ROW_BOTTOM)
							selectedDS.bottomY = Double.parseDouble(str);
						
						notifyMessageListeners("change", "");
					}
					else
					{
						// store the old value
						double revertValue = 0;
						
						if (row == ROW_LEFT)
							revertValue = selectedDS.leftX;
						else if (row == ROW_RIGHT)
							revertValue = selectedDS.rightX;
						else if (row == ROW_TOP)
							revertValue = selectedDS.topY;
						else if (row == ROW_BOTTOM)
							revertValue = selectedDS.bottomY;
						
						// revert the value back to original
						propTable.setValueAt(Double.toString(revertValue), row, 1);
					}
				}
			}
		}
	}
	
	@SuppressWarnings("serial")
	class PropertiesTableCellRenderer extends DefaultTableCellRenderer
	{
		public Component getTableCellRendererComponent(JTable table, Object value,
												boolean isSelected, boolean hasFocus,
												int row, int column)
		{
			// Used to display Boolean and Character
			JLabel label = new JLabel();
			
			// Sets the background color depending on whether the cell
			// is selected or not.
			if (isSelected)
			{
				label.setBackground(table.getSelectionBackground());
				label.setForeground(table.getSelectionForeground());
			}
			else
			{
				label.setBackground(table.getBackground());
				label.setForeground(table.getForeground());
			}
			label.setOpaque(true);

			
			// Create the renderer component depending on type
			if (value instanceof Boolean)
			{
				if (((Boolean)value).booleanValue())
					label.setText("True");
				else
					label.setText("False");
				
				return label; 
			}
			else if (value instanceof Character)
			{
				if (((Character)value).charValue() == 'A')
					label.setText("All");
				else if (((Character)value).charValue() == 'S')
					label.setText("Selected");
				else if (((Character)value).charValue() == 'N')
					label.setText("None");
				
				return label;
			}
			else if (value instanceof Color)
			{				
				return makeColorCell((Color)value, isSelected);
			}
			else
				return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
		}
		
		public JPanel makeColorCell(Color c, boolean isSelected)
		{
			JPanel p = new JPanel(new BorderLayout());
			JPanel box = new JPanel();
			JLabel label = new JLabel();
			label.setText(getHex(c));
			label.setBorder(new EmptyBorder(0, 5, 0, 0));
			
			box.setBorder(BorderFactory.createCompoundBorder(
					BorderFactory.createLineBorder(Color.black, 1),
					BorderFactory.createEmptyBorder(2, 2, 2, 2)));
			
			box.setBackground(c);
			box.setOpaque(true);
			
			if (isSelected)
			{
				p.setBackground(propTable.getSelectionBackground());
				p.setForeground(propTable.getSelectionForeground());
				label.setBackground(propTable.getSelectionBackground());
				label.setForeground(propTable.getSelectionForeground());
			}
			else
			{
				p.setBackground(propTable.getBackground());
				p.setForeground(propTable.getForeground());
				label.setBackground(propTable.getBackground());
				label.setForeground(propTable.getForeground());
			}
			p.setOpaque(true);
			
			p.add(box, BorderLayout.WEST);
			p.add(label, BorderLayout.CENTER);
			
			return p;
		}
		
		private String getHex(Color c)
		{
			int r = c.getRed();
			int g = c.getGreen();
			int b = c.getBlue();
			
			return "" + r + ", " + g + ", " + b;
		}

	}
	
	@SuppressWarnings("serial")
	class PropertiesTableCellEditor extends AbstractCellEditor implements TableCellEditor
	{
		JComponent currentEditor;
		
		public boolean isCellEditable(EventObject e)
		{
			if (e instanceof MouseEvent)
			{
				if (((MouseEvent)e).getClickCount() >= 2)
					return true;
			}
			else if (e == null)
			{
				System.out.println("wtf");
			}
			else
			{
				System.out.println(e.getClass());
			}
			
			return false;
		}

		public Component getTableCellEditorComponent(JTable table, Object value,
														boolean isSelected,
														int row, int column)
		{
			if (value instanceof Boolean)
			{
				currentEditor = new TrueFalseComboBox();
				((TrueFalseComboBox)currentEditor).setBoolean(((Boolean)value).booleanValue());
				return currentEditor;
			}
			else if (value instanceof String)
			{
				currentEditor = new JTextField((String)value);
				currentEditor.setFont(table.getFont());
				return currentEditor;
			}
			else if (value instanceof Color)
			{
				currentEditor = new ColorCellEditor((Color)value);
				return currentEditor;
			}
			else if (value instanceof Character)
			{
				currentEditor = new ModeComboBox();
				((ModeComboBox)currentEditor).setMode((Character)value);
				return currentEditor;
			}
			else
				return new JLabel(value.toString());
		}

		public Object getCellEditorValue()
		{
			if (currentEditor instanceof TrueFalseComboBox)
				return new Boolean(((TrueFalseComboBox)currentEditor).getBoolean());
			else if (currentEditor instanceof JTextField)
				return ((JTextField)currentEditor).getText();
			else if (currentEditor instanceof ColorCellEditor)
				return ((ColorCellEditor)currentEditor).getColor();
			else if (currentEditor instanceof ModeComboBox)
				return ((ModeComboBox)currentEditor).getMode();
			else
				return "";
		}
		
		class TrueFalseComboBox extends JComboBox implements ActionListener
		{
			public TrueFalseComboBox()
			{
				super(new String[] {"True", "False"});
				addActionListener(this);
			}
			
			public void actionPerformed(ActionEvent e)
			{
				fireEditingStopped();
			}
			
			public boolean getBoolean()
			{
				if (((String)getSelectedItem()).equals("True"))
					return true;
				else
					return false;
			}
			
			public void setBoolean(boolean b)
			{
				if (b)
					setSelectedItem("True");
				else
					setSelectedItem("False");
			}
		}
		
		class ModeComboBox extends JComboBox implements ActionListener
		{
			public ModeComboBox()
			{
				super(new String[] {"All", "Selected", "None"});
				addActionListener(this);
			}
			
			public void actionPerformed(ActionEvent e)
			{
				fireEditingStopped();
			}
			
			public Character getMode()
			{
				if (((String)getSelectedItem()).equals("All"))
					return new Character('A');
				else if (((String)getSelectedItem()).equals("Selected"))
					return new Character('S');
				else if (((String)getSelectedItem()).equals("None"))
					return new Character('N');
				else
					return new Character('X');
			}
			
			public void setMode(Character c)
			{
				if (c.charValue() == 'A')
					setSelectedItem("All");
				else if (c.charValue() == 'S')
					setSelectedItem("Selected");
				else if (c.charValue() == 'N')
					setSelectedItem("None");
			}
		}

		
		class ColorCellEditor extends JLabel implements MouseListener
		{
			Color color;
			
			public ColorCellEditor(Color c)
			{
				super(c.toString());
				color = c;
				this.addMouseListener(this);
			}
			
			public Color getColor()
			{
				return color;
			}

			public void mouseClicked(MouseEvent e)
			{
				if (e.getClickCount() >= 1)
				{
					Color c = JColorChooser.showDialog(propTable, "Choose color", color);
					
					if (c != null)
						color = c;
					
					fireEditingStopped();
				}
			}

			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) {}
		}
	}

/*	TODO
 *  -> Enable later when implementing keyboard navigation 
 *  
	public void keyPressed(KeyEvent e) {}
	public void keyReleased(KeyEvent e) {}

	public void keyTyped(KeyEvent e)
	{
		//propTableCellEditor.isCellEditable(e);
	}
*/
}
