//    Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)
//    Copyright (C) 2008 Chris Kroells
//
//    This file is part of Dvaspi.
//
//    Dvaspi is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Dvapsi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.


package net.coobird.dvaspi;

import java.awt.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import java.util.*;

import javax.swing.*;
import javax.swing.table.*;

/**
 * Displays the selected data points of a given data set in a table,
 * and allows the deselection of points. The selected data points can
 * also be copied to the clipboard.
 * @author coobird
 */
@SuppressWarnings("serial")
public class DvaspiSelectionTable extends JPanel implements ActionListener 
{
	private JTable selectionTable;
	private NonEditableTableModel selectionTableModel;
	private JScrollPane selectionTableScrollPane;
	private JPanel buttonPanel;
	private JButton selectionTableDeselectButton;
	private JButton selectionTableCopyButton;
	private DataSet isolatedDS;
	private Vector<MessageListener> messageListeners;

	
	public DvaspiSelectionTable()
	{
		super(new BorderLayout());
		selectionTable = new JTable();
		selectionTableModel = new NonEditableTableModel();
		selectionTableModel.setColumnIdentifiers(new String[] {"Index", "X", "Y"});
		selectionTable.setModel(selectionTableModel);
		this.setName("Selection Table");
		selectionTableScrollPane = new JScrollPane(selectionTable);
		this.add(selectionTableScrollPane, BorderLayout.CENTER);
		buttonPanel = new JPanel(new BorderLayout());
		selectionTableDeselectButton = new JButton("Deselect");
		selectionTableDeselectButton.addActionListener(this);
		selectionTableCopyButton = new JButton("Copy");
		selectionTableCopyButton.addActionListener(this);
		buttonPanel.add(selectionTableDeselectButton, BorderLayout.NORTH);
		buttonPanel.add(selectionTableCopyButton, BorderLayout.SOUTH);
		this.add(buttonPanel, BorderLayout.PAGE_END);
		
		messageListeners = new Vector<MessageListener>();
		
		validate();
	}
	
	/**
	 * Sets the DataSet object currently being processed.
	 * @param ds DataSet to be processed.
	 */
	public void setIsolatedDS(DataSet ds)
	{
		isolatedDS = ds;
	}
	
	/**
	 * Returns the DataSet object currently being processed.
	 * @return DataSet object currently being processed.
	 */
	public DataSet getIsolatedDS()
	{
		return isolatedDS;
	}
	
	/**
	 * Clears the DataSet currently being processed.
	 */
	public void clearIsolatedDS()
	{
		isolatedDS = null;
	}

	public void addMessageListener(MessageListener l)
	{
		messageListeners.add(l);
	}
	
	public void removeMessageListener(MessageListener l)
	{
		messageListeners.remove(l);
	}
	
	/**
	 * Notifies the registered MessageListeners of an event.
	 * This method prepares an MessageEvent object which is sent to
	 * all registered Message Listeners.
	 * @param action Action that is performed.
	 * @param msg Message that accompanies the action.
	 */
	private void notifyMessageListeners(String action, String msg)
	{
		for (Enumeration<MessageListener> e = messageListeners.elements(); e.hasMoreElements(); )
			e.nextElement().messageReceived(new MessageEvent(this, action, msg));
	}

	
	@SuppressWarnings("serial")
	class NonEditableTableModel extends DefaultTableModel
	{
		public boolean isCellEditable(int arg0, int arg1)
		{
			// No cells are editable!
			return false;
		}
	}
	
	/**
	 * Updates the Selection Table.
	 * @return Number of selected points.
	 */
	public int updateSelectedTable(DataSet ds)
	{
		if (ds == null)
		{
			this.removeAll();
			this.add(new JLabel("No data set selected."), BorderLayout.PAGE_START);
			return -1;
		}
		else
		{
			if (!this.isAncestorOf(selectionTableScrollPane))
			{
				this.removeAll();
				this.add(selectionTableScrollPane, BorderLayout.CENTER);
				this.add(buttonPanel, BorderLayout.PAGE_END);
			}
		}		
		isolatedDS = ds;
		
		// Does copying the array locally really speed things up??
		double x[] = new double[ds.x.length];
		System.arraycopy(ds.x, 0, x, 0, ds.x.length);
		double y[] = new double[ds.y.length];
		System.arraycopy(ds.y, 0, y, 0, ds.y.length);
		boolean sb[] = new boolean[ds.selected.length];
		System.arraycopy(ds.selected, 0, sb, 0, ds.selected.length);
		
		int count = 0;
		
		// Counts the number of selected points
		for (int i = 0; i < sb.length; i++)
			if (sb[i])
				count++;
		
		// Data 2d-array used to create the selection table
		Double data[][] = new Double[count][3];
		
		// Populates the 2d-array with selected points
		for (int i = 0, c = 0; i < sb.length; i++)
		{
			if (sb[i])
			{
				data[c][0] = new Double(i);
				data[c][1] = new Double(x[i]);
				data[c][2] = new Double(y[i]);
				c++;
			}
		}
		
		// Column titles
		String names[] = {"Index", "X", "Y"};
		
		selectionTableModel.setDataVector(data, names);
		return count;
	}


	/**
	 * 
	 */
	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		
		notifyMessageListeners("needds", "needds");
		
		if (source == selectionTableDeselectButton)
		{
			// Currently selected rows in the Selection Table, intended to be removed.
			int[] row = selectionTable.getSelectedRows();
			
			if (row == null)
				return;
			
			// Get current dataset
			DataSet ds = isolatedDS;
			
			// Removes the rows from table, and deselects from the DataSet
			for (int i = 0; i < row.length; i++)
			{
				Double indexDouble = (Double)selectionTable.getModel().getValueAt(row[i], 0);
				int index = indexDouble.intValue();
				ds.selected[index] = false;
				notifyMessageListeners("select", "Deselected point at index " + index);
			}
			updateSelectedTable(isolatedDS);

			notifyMessageListeners("drawonce", "drawonce");
		}
		else if (source == selectionTableCopyButton)
		{
			int[] row = selectionTable.getSelectedRows();
			
			if (row == null)
				return;
			
			// Get current dataset
			DataSet ds = isolatedDS;
			
			String s = "";
			
			// Removes the rows from table, and deselects from the DataSet
			for (int i = 0; i < row.length; i++)
			{
				Double indexDouble = (Double)selectionTable.getModel().getValueAt(row[i], 0);
				int index = indexDouble.intValue();
				s += ds.x[index] + "\t" + ds.y[index] + "\n";
			}
			
			// Copy the data points to the clipboard
			try
			{
				Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
				cb.setContents(new StringSelection(s), null);
				
				notifyMessageListeners("copy", s);
			}
			catch (Exception ex)
			{
				notifyMessageListeners("showerror", "Could not copy data points to the clipboard.");
			}
		}		
	}
}