//    Coobird's Data Viewing and Selecting Program with Integration (Dvaspi)
//    Copyright (C) 2008 Chris Kroells
//
//    This file is part of Dvaspi.
//
//    Dvaspi is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Dvapsi is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Dvaspi.  If not, see <http://www.gnu.org/licenses/>.


package net.coobird.dvaspi;

import java.util.*;
import java.awt.*;
import javax.swing.JPanel;

/**
 * Provides a visual overview of the data sets' limits along with
 * the current viewport.
 * @author coobird
 * 
 */
@SuppressWarnings("serial")
public class DvaspiWindowPanel extends JPanel
{
	private Vector<DataSet> dsVector;
		
	private double minX = 0;
	private double maxX = 0;
	private double minY = 0;
	private double maxY = 0;
		
	
	public DvaspiWindowPanel()
	{
		this.setBackground(Color.white);
	}
	
	public void setDataSetVector(Vector<DataSet> dsv)
	{
		dsVector = dsv;
	}

	private void calcLimits()
	{
		Enumeration<DataSet> dsEnum = dsVector.elements();
		DataSet ds;
		
		// Retrieve the first element, set that as the initial limits
		if (dsEnum.hasMoreElements())
		{
			ds = dsEnum.nextElement();
			
			minX = ds.absLeftX;
			maxX = ds.absRightX;
			minY = ds.absTopY;
			maxY = ds.absBottomY;

			if (ds.leftX < minX)
				minX = ds.leftX;
			if (ds.rightX > maxX)
				maxX = ds.rightX;
			if (ds.topY < minY)
				minY = ds.topY;
			if (ds.bottomY > maxY)
				maxY = ds.bottomY;		
		}
		else
		{
			return;
		}
		
		while (dsEnum.hasMoreElements())
		{
			ds = dsEnum.nextElement();
			
			if (ds.absLeftX < minX)
				minX = ds.absLeftX;
			if (ds.absRightX > maxX)
				maxX = ds.absRightX;
			if (ds.absTopY < minY)
				minY = ds.absTopY;
			if (ds.absBottomY > maxY)
				maxY = ds.absBottomY;
			if (ds.leftX < minX)
				minX = ds.leftX;
			if (ds.rightX > maxX)
				maxX = ds.rightX;
			if (ds.topY < minY)
				minY = ds.topY;
			if (ds.bottomY > maxY)
				maxY = ds.bottomY;
		}
	}
	
	
	public void paintComponent(Graphics g)
	{
		int panelWidth = this.getWidth();
		int panelHeight = this.getHeight();

		// Be sure the OR expression is short-circuited. Else a NullPointerException
		// will be thrown when dsVector is null.
		if (dsVector == null || dsVector.isEmpty())
		{
			g.setColor(Color.black);
			
			int textWidth = g.getFontMetrics().stringWidth("No data set loaded.");
			int centerX = (panelWidth / 2) - (textWidth / 2);
			int centerY = panelHeight / 2;
			
			g.drawString("No data set loaded.", centerX, centerY);
			
			return;
		}
		
		g.setColor(Color.white);
		g.fillRect(0,0,panelWidth,panelHeight);
		
		calcLimits();
		
		// Calculate the Window dimensions
		double winWidth = maxX - minX;
		double winHeight = maxY - minY;
		
		// Display area will have a 10% margin
		winWidth *= 1.1;
		winHeight *= 1.1;

		// Scaling code, same as DvaspiPanel
		double centerx = (maxX + minX) / 2;
		double centery = (maxY + minY) / 2;
		
		double xfactor = panelWidth / winWidth;
		double yfactor = panelHeight / winHeight;
		
		for (Enumeration<DataSet> dsEnum = dsVector.elements(); dsEnum.hasMoreElements(); )
		{
			DataSet ds = dsEnum.nextElement();
			
			int offsetx = (int)Math.round(((winWidth / 2) - centerx) * xfactor);
			int offsety = (int)Math.round(((winHeight / 2) - centery) * yfactor);
			
			// Limits for the data set points
			int leftX = (int)Math.round(ds.absLeftX * xfactor) + offsetx;
			int rightX = (int)Math.round(ds.absRightX * xfactor) + offsetx;
			int topY = (int)Math.round(ds.absTopY * yfactor) + offsety;
			int bottomY = (int)Math.round(ds.absBottomY * yfactor) + offsety;
			// Limits for the current window for that data set
			int wLeftX = (int)Math.round(ds.leftX * xfactor) + offsetx;
			int wRightX = (int)Math.round(ds.rightX * xfactor) + offsetx;
			int wTopY = (int)Math.round(ds.topY * yfactor) + offsety;
			int wBottomY = (int)Math.round(ds.bottomY * yfactor) + offsety;
			
			int myWidth = rightX - leftX;
			int myHeight = bottomY - topY;
			int myWWidth = wRightX - wLeftX;
			int myWHeight = wBottomY - wTopY;
			
			
			//System.out.println("x:" + leftX + " y:" + topY + " width:" + myWidth + " height:" + myHeight);
			Color c = new Color(ds.lineColor.getRed(), ds.lineColor.getGreen(), ds.lineColor.getBlue(), 64);
			Color c2 = new Color(ds.lineColor.getRed(), ds.lineColor.getGreen(), ds.lineColor.getBlue(), 128);
			
/*			// Draw area where points of data set exists as a filled box
			g.setColor(c);
			g.fillRect(leftX, topY, myWidth, myHeight);
			// Draw the current window for the data set as a solid line box
			g.setColor(c2);
			g.drawRect(wLeftX, wTopY, myWWidth, myWHeight);
*/			

			// Draw area where points of data set exists as a filled box
			g.setColor(c2);
			g.drawRect(leftX, topY, myWidth, myHeight);
			// Draw the current window for the data set as a solid line box
			g.setColor(c);
			g.fillRect(wLeftX, wTopY, myWWidth, myWHeight);
			

			
			// Show the title of data set
			g.setColor(ds.lineColor);
			g.drawString(ds.getName(), leftX, topY);
		}
	}
}
